# -*- coding: utf-8 -*-
# http://code.activestate.com/recipes/473786-dictionary-with-attribute-style-access/


class AttrDict(dict):
    """A dictionary with attribute-style access. It maps attribute access to
    the real dictionary.  """

    def __init__(self, init={}):
        dict.__init__(self, init)

    def __getstate__(self):
        return self.__dict__.items()

    def __setstate__(self, items):
        for key, val in items:
            self.__dict__[key] = val

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__, dict.__repr__(self))

    def __setitem__(self, key, value):
        return super(AttrDict, self).__setitem__(key, value)

    def __getitem__(self, name):
        return super(AttrDict, self).__getitem__(name)

    def __delitem__(self, name):
        return super(AttrDict, self).__delitem__(name)

    __getattr__ = __getitem__
    __setattr__ = __setitem__

    def copy(self):
        ch = AttrDict(self)
        return ch



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Count Lines
    ---
    count_lines - count the number of lines in a text file

    AutoSys
    ---
    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """

# 'Standard Library'
from os import argv
from pathlib import Path

# 'package imports'
from autosys.file_ops import codec_open

# (venv) ➜  autosys (dev) ✗)python3 -c "import sys; print(f'{sys.argv}'); n=0; with open(sys.argv[1]): n += 1; print(n);" 'pip list'



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2007-2018 The Python-Twitter Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""A library that provides a Python interface to the Twitter API."""
# 'future imports'
from __future__ import absolute_import

# 'Standard Library'
import json  # noqa

# 'local imports'
from ._file_cache import _FileCache  # noqa
from .api import Api  # noqa
from .error import TwitterError  # noqa
from .models import DirectMessage  # noqa
from .models import Hashtag  # noqa
from .models import List  # noqa
from .models import Media  # noqa
from .models import Status  # noqa
from .models import Trend  # noqa
from .models import Url  # noqa
from .models import User  # noqa
from .models import UserStatus  # noqa
from .parse_tweet import ParseTweet  # noqa

from .models import Category  # noqa; noqa

__author__ = "The Python-Twitter Developers"
__email__ = "python-twitter@googlegroups.com"
__copyright__ = "Copyright (c) 2007-2016 The Python-Twitter Developers"
__license__ = "Apache License 2.0"
__version__ = "3.5"
__url__ = "https://github.com/bear/python-twitter"
__download_url__ = "https://pypi.python.org/pypi/python-twitter"
__description__ = "A Python wrapper around the Twitter API"



try:
    from hashlib import md5  # noqa
except ImportError:
    from md5 import md5  # noqa



# 'Standard Library'
from pathlib import Path


def read(filename: str, ignore_errors: bool = False):
    """ Returns the text of `filename`.

        filename: str       - name of file to read and return
        ignore_errors: bool - ignore all errors and return None instead
        """
    log.info(f"function `read` with args {filename=} and {ignore_errors=}.")
    try:
        file_path = Path(__file__).resolve().parents[0] / filename
        log.info(f"read from: {file_path}")  # debug
        with open(file_path, encoding=DEFAULT_ENCODING) as f:
            return f.read()
    except (OSError, NameError, TypeError) as e:
        if not ignore_errors:
            return e
    return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" link manager - maintain symlinks between home and backup (~/.dotfiles)

    ---
    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """


import re
import os
import sys

from dataclasses import Field, dataclass, field
from enum import Enum, IntEnum
from functools import lru_cache
from locale import getpreferredencoding
from pathlib import Path
import platform
from io import TextIOWrapper

from autosys.text_utils.datetime import datetime as dt

from typing import Dict, List, Tuple

try:
    import ujson as to_json
except ImportError:
    import json

try:
    import lxml
    DEFAULT_PARSER = lxml
    XML_PARSER = lxml
except ImportError:
    XML_PARSER = False
    try:
        import html5lib
        DEFAULT_PARSER = html5lib
    except ImportError:
        import html
        DEFAULT_PARSER = html

try:
    import numpy as np
except ImportError:
    np = False


try:
    from autosys import logging
    log = logging.getLogger(__file__)
    USE_DEFAULT_LOGGER: bool = False  # if logging is not available
except:
    USE_DEFAULT_LOGGER: bool = True  # if logging is not available

obj = object()
Nul: str = chr(0)


# ? ========================= Base Class


@dataclass
class _GenericDataClass:

    ''' Base Class for most of *my* dataclass objects. Allows custom
        reporting, logging, etc to be designed once and to therefore offer
        consistent methods and interfaces.
        '''

    def default_logger(self, msg):
        ''' A generic stderr logger for use when logging fails. '''
        print(f" <default_logger> {msg}", file=sys.stderr)

    def info(self, msg):
        try:
            logging.info(msg=msg)
        except:
            if USE_DEFAULT_LOGGER:
                self.default_logger(msg=msg)

    def breakpointhook(self):
        sys.breakpointhook()

    def refcount(self, obj):
        return sys.getrefcount(obj)

    @property
    def myclass(self):
        return str(self.__class__).split('.')[1][:-2]

    def pulse(self, obj):
        ''' log a snapshot of the object's state. '''
        return f"class: {obj.myclass} - mem: {obj.__sizeof__()}"

    def size(self, obj):
        sys.getsizeof(obj)

    def log_pulse(self, obj):
        info(pulse(obj=obj))

    def pprint(self):
        self.pulse(self)
        print('='*50)
        print(f"{self.pulse(self)}")
        for k, v in self.__dict__.items():
            print(f"  {k:<20.20} : {repr(v):<50.30}")
        print('-'*50)

# ? ========================= Defaults


@dataclass
class _FileSystemDefaults(_GenericDataClass):
    ''' Generic Defaults for any system. Not intended to be used directly, but
    rather to create a specific defaults sub-class, like 'macOS' or 'Win32'
    '''
    BYTEORDER: str = sys.byteorder
    CPU_COUNT: int = os.cpu_count()
    NL: str = os.linesep
    PLAT: str = platform.platform()
    STDERR: TextIOWrapper = sys.stderr
    STDIN: TextIOWrapper = sys.stdin
    STDOUT: TextIOWrapper = sys.stdout

    ARGV: List = field(init=False)
    PY3: bool = field(init=False)
    PYTHONPATH: List = field(init=False)
    PYVER: Tuple = field(init=False)

    try:
        DEFAULT_ENCODING: str = getpreferredencoding(
            do_setlocale=True)
    except:
        DEFAULT_ENCODING: str = "utf-8"

    def __post_init__(self):
        self.ARGV = sys.argv
        self.PYTHONPATH = sys.path
        self.PYVER: Tuple = sys.version_info
        self.PY3: bool = self.PYVER[0] > 2


@dataclass
class PosixDefaults(_FileSystemDefaults):
    DEFAULT_FILE_PERMISSIONS: int = 664
    DEFAULT_DIRECTORY_PERMISSIONS: int = 775


@dataclass
class LinuxDefaults(PosixDefaults):
    ADD_ON_INSTALLER: str = 'apt-get'


@dataclass
class macOSDefaults(PosixDefaults):
    ADD_ON_INSTALLER: str = 'brew'


@dataclass
class WinDefaults(_FileSystemDefaults):
    ''' Includes windows specific defaults.

        Examples:
        - file separator character
        - newline character
        '''


MACHINE: str = platform.platform().lower()

try:
    assert (WIN32 := 'win' in MACHINE)
    import ntsecuritycon
    import win32security
    defaults = WinDefaults()
except AssertionError:
    try:
        assert (MACOS := 'macos' in MACHINE)
        defaults = macOSDefaults()
    except AssertionError:
        defaults = LinuxDefaults()


# ? ========================= Types and Tags


class WordLink:
    ''' a connection in a graph '''
    linkid: int
    date_added: dt
    date_updated: dt
    file_set: str  # a set of connected words


class Graph:
    graphid: int


class Word:
    word_id: int
    date_added: dt
    date_updated: dt
    name: str
    linkset: Graph  # a set of connections to other words


class FileTag:
    ''' a topic tag that is attached to items where it is related.

        a 'weighting' or 'percentage' of 'something(??)' should be included
        with the link to document the actual connection and keep the two
        items' properties separate
        '''
    name: str
    description: str
    link: str  # Wikipedia, etc.
    graphnodes: Graph  # a set of connections to other FileTags
    keywords: WordLink  # a set of relationships to words


class ItemTypes(IntEnum):
    file: int = 1
    link: int = 2
    directory: int = 3
    volume: int = 4
    url: int = 5
    stream: int = 6

# ? ========================= Containers


@dataclass
class _GenericContainerItem(_GenericDataClass):
    ''' A basic "file system object" that includes anything that can be
        accessed from a file system request, stream, socket, etc.
        '''
    file_name: str      # name of resource
    url: str = ''       # location of resource
    kind: ItemTypes = ItemTypes.file  # type of resource
    protected: bool = True
    encrypted: bool = False
    encryption_algoritm: Field = field(init=False)


@dataclass
class _StorageItem(_GenericContainerItem):
    ''' A generic storage item ... meaning an item that is stored on a disk
        and can be accessed with the pathlib library. Most 'file' commands
        work on these items. e.g. stat, open, ls

        It could be a 'model' of a container that is used as an interface to
        another device. e.g. an adapter to access a database with file
        commands ...
        '''
    _stat: Field = field(init=False)  # file system stats (on demand)

    def __post_init__(self):
        self.file_name = self.me.resolve()

    @ property
    def me(self):
        return Path(self.file_name)

    def chmod(self, mode: int = 664):
        self.me.chmod(mode=mode)

    def stat(self):
        self._stat = self.me.stat()
        return self._stat

    def isfile(self) -> (bool):
        return self.me.is_file()

    def isdir(self) -> (bool):
        return self.me.is_dir()

    def islink(self) -> (bool):
        return self.me.is_symlink()

    @ property
    def realpath(self):
        return self.me.resolve()

    @ property
    def parent(self):
        return self.me.parent()

    @ property
    def parents(self):
        return self.me.parents()


class Directory(_GenericContainerItem):
    ''' A directory on a file system ... or equivalent.

        Mainly used as a container for files and links ... '''
    kind: ItemTypes = ItemTypes.file  # type of resource

    def chmod(self, mode: int = 775):
        self.me.chmod(mode=mode)


class SymLink(_StorageItem):
    ''' A symlink to a file on the file system. '''

    def stat(self):
        self._stat = self.me.stat()
        return self._stat

    def chmod(self, mode: int = 664):
        self.me.lchmod(mode=mode)


@dataclass
class DataFile(_StorageItem):
    ''' A generic data file. This can be any file that is 'readable' in some
        standard way. This is a file that can be access with the os.stat or
        equivalent command.

        Could be text or bytes ... basically, not a compiled executable.

        The stat and tags are generated on demand or when triggered.
        '''
    _tags: Field = field(init=False)  # a list of related keywords


@dataclass
class TextFile(DataFile):
    data: str = field(init=False)  # loaded as needed
    # a record of the main source
    main_source: str = field(init=False)
    # derived from the text or topic/type
    main_topic: str = field(init=False)
    # a measure of social media pressense ???
    presence: int = field(init=False)
    # a list of related sources
    sources: Field = field(init=False)

    def to_json(self):
        pass

    def to_markdown(self):
        pass

    def to_csv(self):
        pass

    def to_html(self):
        pass

    def to_man(self):
        pass

    def to_doc(self):
        pass


@dataclass
class Folder(_StorageItem):
    _storage: Field = field(init=False)
    _index: Field = field(init=False)

    def __post_init(self):
        self.parse()

    def parse(self):
        ''' Parse the directory tree and collect data.

            This MUST be done when a new directory is indexed.

            After that, individual files requested can be read as needed.

            This may also be done periodically to keep information fresh. The
            frequency depends on whether other programs access the directory
            and how often changes are made.
            '''
        pass

    def _add(self, file_name):
        ''' Add a file to this folder. '''
        pass

    def _del(self, file_name):
        ''' Delete a file from this Folder. '''
        pass

    def _stat(self, file_name):
        ''' Get file information. '''
        pass

    def _copy(self, from_file, to_file):
        ''' Copy a file to another Folder. '''
        pass

    def _move(self, from_spot, to_spot):
        ''' Move a file to another Folder. '''
        pass


home_list: List = []
dotfiles_list: List = []

defaults.pprint()

tmp = _GenericContainerItem('testfile')

tmp.pprint()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" AutoSys Package Metadata
    ---
    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """

from setup import pip_safe_name, readme

from typing import Dict, List, Tuple

__version__: str = '0.4.4'


NAME: str = pip_safe_name("AutoSys")

VERSION: str = __version__  # "0.4.4"
VERSION_INFO: Tuple[int] = VERSION.split(".")
DESCRIPTION: str = "System utilities for Python on macOS."
REQUIRES_PYTHON: str = ">=3.8.0"
PACKAGE_DIR: Dict = {'': f'{NAME}'}
PACKAGE_EXCLUDE: List[str] = ['*test*', '*bak*']
LICENSE: str = "MIT"
LONG_DESCRIPTION: str = readme()
LONG_DESCRIPTION_CONTENT_TYPE: str = "text/markdown"
# LONG_DESCRIPTION_CONTENT_TYPE="text/x-rst",
AUTHOR: str = "Michael Treanor"
AUTHOR_EMAIL: str = "skeptycal@gmail.com"
MAINTAINER: str = ""
MAINTAINER_EMAIL: str = ""
URL: str = f"https://skeptycal.github.io/{NAME}/"
DOWNLOAD_URL: str = f"https://github.com/skeptycal/{NAME}/archive/{VERSION}.tar.gz"
ZIP_SAFE: bool = False
INCLUDE_PACKAGE_DATA: bool = True
# What packages are required for this module to be executed?
REQUIRED: List[str] = [
    "aiocontextvars>=0.2.0 ; python_version<'3.7'",
    "colorama>=0.3.4 ; sys_platform=='win32'",
    "win32-setctime>=1.0.0 ; sys_platform=='win32'",
]

# What packages are optional?
EXTRAS: Dict = {
    ":python_version < '3.5'": ["typing==3.6.1", ],
    "dev": [
        "black>=19.3b0 ; python_version>='3.8'",
        "codecov>=2.0.15",
        "flake8>=3.7.7",
        "isort>=4.3.20",
        "tox>=3.9.0",
        "tox-travis>=0.12",
        "pytest>=4.6.2",
        "pytest-cov>=2.7.1",
        "Sphinx>=2.2.1",
        "sphinx-autobuild>=0.7.1",
        "sphinx-rtd-theme>=0.4.3",
    ]
}

PACKAGE_DATA: Dict = {
    # If any package contains these files, include them:
    "": [
        "*.txt",
        "*.rst",
        "*.md",
        "*.ini",
        "*.png",
        "*.jpg",
        "*.py",
        "__init__.pyi",
        "py.typed",
    ]
}

PROJECT_URLS: Dict = {
    "Website": f"https://skeptycal.github.io/{NAME}/",
    "Documentation": f"https://skeptycal.github.io/{NAME}/docs",
    "Source Code": f"https://www.github.com/skeptycal/{NAME}/",
    "Changelog":
    f"https://github.com/skeptycal/{NAME}/blob/master/CHANGELOG.md",
}

KEYWORDS: List = [
    "application",
    "macOS",
    "dev",
    "devops",
    "cache",
    "utilities",
    "cli",
    "python",
    "cython",
    "text",
    "console",
    "log",
    "debug",
    "test",
    "testing",
    "logging",
    "logger",
]
CLASSIFIERS: List = [
    "Development Status :: 4 - Beta",
    "License :: OSI Approved :: MIT License",
    "Environment :: Console",
    "Environment :: MacOS X",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "Operating System :: MacOS",
    "Operating System :: OS Independent",
    "Programming Language :: Cython",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3 :: Only",
    "Programming Language :: Python :: 3",
    # These are the Python versions tested; it may work on others
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Software Development :: Testing",
    "Topic :: Utilities",
]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_www.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal
# from __future__ import absolute_import, print_function

# 'Standard Library'
import os
import sys

# 'third party'
import requests

CONNECT_ERRORS = (
    ConnectionError,
    ConnectionAbortedError,
    ConnectionResetError,
    ConnectionRefusedError,
)


def url_test(url: str = "http://www.google.com") -> (int):
    """
    Return response from server request.
    """
    try:
        r = requests.get(url)
    except CONNECT_ERRORS as e:
        print(f"Connection error: {e}")
        return 0
    return r.status_code


if __name__ == "__main__":

    print()
    urls = [
        "http://www.google.com",
        "https://www.google.com",
        "https://www.twitter.com/skeptycal",
    ]

    # 'https://www.skeptycal.com'
    for url in urls:
        res = url_test(url)
        print()
        if res == 200:
            print(f"Successful connection to {url} ...response: {res}")
        else:
            print(f"No connection to {url} ... response: {res}")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" logical_xor.py """
# copyright (c) 2020 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal


def logical_xor_so(a, b):
    """ Return True or False based on the truth table below

        | a | b  | xor   |             |
        |---|----|-------|-------------|
        | T | T  | F     |             |
        | T | F  | T     | a and not b |
        | F | T  | T     | not a and b |
        | F | F  | F     |             |

        Python logical or: A or B:
            returns A if bool(A) is True, otherwise returns B
        Python logical and: A and B:
            returns A if bool(A) is False, otherwise returns B

        https://stackoverflow.com/a/432901
        """
    if bool(a) == bool(b):
        return False
    else:
        return a or b


def logical_xor(a, b):
    """ Return True or False based on the truth table below

        | a | b  | xor   |             |
        |---|----|-------|-------------|
        | T | T  | F     |             |
        | T | F  | T     | a and not b |
        | F | T  | T     | not a and b |
        | F | F  | F     |             |

        Python logical or: A or B:
            returns A if bool(A) is True, otherwise returns B
        Python logical and: A and B:
            returns A if bool(A) is False, otherwise returns B
        """
    if a == b:
        return False
    else:
        return a or b


def xor(a, b):
    return (a and not b) or (not a and b)


class Int2(int):
    def __new__(cls, value):
        return super().__new__(cls, value + 42)

    def __xor__(self, other):
        a = self
        b = other
        return 42

    # def __str__(self):
    #     return str(self.value + 42)
    # lambda a, b: (a and not b) or (not a and b)


a = Int2(3)
b = 3
c = 4
d: Int2 = 4

print(d)
print(type(d))
print(dir(d))
print(d ^ 1)

print(a, bool(a), id(a))
print(b, bool(b), id(b))
print(c, bool(c), id(c))

print()
print(logical_xor_so(a, b))
print(logical_xor(a, b))
print(xor(a, b))
print(a ^ b)
print()
print(logical_xor_so(a, c))
print(logical_xor(a, c))
print(xor(a, c))
print()
print(logical_xor_so(c, b))
print(logical_xor(c, b))
print(xor(c, b))



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
""" prof_json """

# TODO -- list of things to profile
#   _: List[Any] = []
#       create new list upon entering function
#   vs  use standard scratch list and clear upon entering
#   vs  numpy array
#   vs  dict, set, frozenset, tuple ...

if True:
    import os
    from os.path import dirname
    import shutil

    # import sys
    # import tempfile
    from tempfile import NamedTemporaryFile, TemporaryFile, mkstemp

    # from contextlib import contextmanager
    from functools import lru_cache
    from time import perf_counter_ns as perf
    from typing import *
    import multiprocessing
    import threading
    from cli.terminal import br

    # import numpy as np

if True:  # temp file setup
    TMP_SUFFIX: str = ".log"
    TMP_PREFIX: str = "profile_"
    HERE: str = dirname(__file__)

    TMP_NAMED: str = NamedTemporaryFile()  # e.g. with TMP_NAMED as f:
    TMP: str = TemporaryFile()  # e.g. with TMP as f:

    # temp file in current directory using defaults
    TMP_HERE = mkstemp(
        prefix=TMP_PREFIX, suffix=TMP_SUFFIX, dir=HERE, text=True
    )


if True:  # json modules to test
    import json as json1
    import ujson as json2
    import rapidjson as json3
    import orjson as json4


if True:  # constants
    EVAL_PRECOMPILED = compile("2", "eval.log", "eval")


def cpu_usable():
    return multiprocessing.cpu_count()


def thread_count():
    return threading.activeCount()


class Code:
    """
    name
    number of varieties
    eval code for varieties
    number of loops defined
    number of changes defined
    input datasets preloaded
    timer variable preset
    """

    def __init__(self):
        self.name = "name"
        self.code: str = code
        self.compiled = compile(
            source=self.code,
            filename="tmp_profile.log",
            mode="eval",
            optimize=True,
        )
        super().__init__()

    pass


def profile():

    # setup reps and trials
    # setup data changes
    # start timer
    # loops
    # call profile code
    # catch timer
    pass


def load_profile(names: List[str]) -> Dict[str, str]:
    pass


@lru_cache()
def eval_int():
    return eval("2")
    # return eval(EVAL_PRECOMPILED)


@lru_cache()
def set_int():
    return int("2")


@lru_cache()
def eval_test(n: int = 1000):
    dt: int = 0
    tt: int = 0
    mint: int = 100000000
    maxt: int = 1
    tt = perf()
    for _ in range(n):
        dt = perf()
        eval_int()
        dt = perf() - dt
        if dt < mint:
            mint = dt
        if dt > maxt:
            maxt = dt
    tt = (perf() - tt) // n
    return tt, mint, maxt


@lru_cache()
def set_test(n: int = 1000):
    dt: int = 0
    tt: int = 0
    mint: int = 100000000
    maxt: int = 1
    tt = perf()
    for _ in range(n):
        dt = perf()
        set_int()
        dt = perf() - dt
        if dt < mint:
            mint = dt
        if dt > maxt:
            maxt = dt
    tt = (perf() - tt) // n
    return tt, mint, maxt


def print_json_module_info():
    for i in range(1, 5):
        name = eval("json{}.__name__".format(i))
        version = eval("json{}.__version__".format(i))
        # print(fmt_str1, fmt_str2)
        fmt_str = f"{name:15.15}.......{version:>10.10}"
        print(fmt_str)


def main():
    """
    CLI script main entry point.
    """
    with TMP_NAMED as LOGFILE:

        print(LOGFILE.name)
        print(f"{multiprocessing.cpu_count()=}")
        print(f"{os.cpu_count()=}")
        print(f"{thread_count()=}")

        br()
        print("\n", "*" * 50, "\n")

        num: int = 0
        width: int = 66
        data: np.array()
        FMT_2_TEST: str = "{:<2} |  {:<10} | {:>10} {:>10} | {:>10}  {:>10}"
        print("_" * width)
        print(
            FMT_2_TEST.format(
                "i", "n", "eval   avg", "min", "set    avg", "min"
            )
        )
        print("-" * width)
        for i in range(0, 8):
            num = int(5 ** i)
            e, min_e, max_e = eval_test(num)
            s, min_s, max_s = set_test(num)

            print(FMT_2_TEST.format(i, num, e, min_e, s, min_s))
        print()
        print(f"{os.cpu_count()=}")
        print(f"{shutil.get_terminal_size()=}")


if __name__ == "__main__":  # if script is loaded directly from CLI
    main()

"""
Average times comparing two functions:

    int('2') # convert string to int

        vs.

    eval('2') # evaluate integer 2
        (pre-compiled vs. live)

Notes about testing setup:

    I started by looping through integers and using 10 ^ i as the number of loops, but the performance increased too quickly and some of the last loops were too slow

    I ended up raising 5 to powers of 0 through 8 and achieved reasonably quick results with some obvious differences in performance.So, the best fit for this test, on this system, pairing an obvious trend with reasonable times are:

        i = [0 .. 8]
        n = 5 ** i

Methods:

    Times are recorded as ∆t with t(0) and t(1) setup immediately before and after the single statement being tested, respectively. They are measured in integer nanoseconds using the 'perfcounter' function from 'time.'

    A few sample tests with other counters showed this counter to be the most consistent, but there were not any large discrepancies in any event. I am no expert in 'timing' or 'profiling' and I am making this up as I go, so ymmv ...

_____________________________________________________________________

Data Set with precompiled eval statement, EVAL_PRECOMPILED, used in place of a live eval.

- lrucache: 0   eval: precompiled

__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |       4267       2100 |       3505        2631
1  |  5          |        900        377 |       1170         382
2  |  25         |        638        347 |        609         347
3  |  125        |        607        352 |        594         349
4  |  625        |        708        352 |        926         366
5  |  3125       |        801        322 |       1078         341
6  |  15625      |        770        322 |        695         334
7  |  78125      |        762        321 |        951         332

- lrucache: loops only   eval: precompiled

__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |       5577       2724 |       4786        3449
1  |  5          |       1711        644 |       1492         655
2  |  25         |       1202        618 |       2968         544
3  |  125        |       1229        590 |       1399         556
4  |  625        |        882        351 |        858         335
5  |  3125       |        606        318 |        937         323
6  |  15625      |       1055        320 |        993         331
7  |  78125      |       1060        317 |        841         313

- lrucache: tests only   eval: precompiled

__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |       5701       3787 |       5077        4257
1  |  5          |        608        175 |        540         184
2  |  25         |        444        178 |        429         174
3  |  125        |        437        175 |        513         179
4  |  625        |        409        174 |        410         176
5  |  3125       |        467        164 |        551         186
6  |  15625      |        543        164 |        720         169
7  |  78125      |        504        164 |        538         166

- lrucache: all 4   eval: precompiled

__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |       3981       2303 |       3447        2513
1  |  5          |        607        170 |        514         174
2  |  25         |        422        173 |        428         179
3  |  125        |        400        175 |        399         175
4  |  625        |        406        173 |        404         166
5  |  3125       |        431        171 |        428         168
6  |  15625      |        479        163 |        669         165
7  |  78125      |        474        162 |        772         166

- lrucache: 0   eval: live

__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |      20787      18892 |       3665        2534
1  |  5          |       8374       6334 |        814         339
2  |  25         |       6590       6125 |        631         354
3  |  125        |       6520       6104 |        604         342
4  |  625        |       7448       5870 |        996         354
5  |  3125       |       8635       5875 |        592         325
6  |  15625      |       8280       5710 |        648         315
7  |  78125      |       7231       5511 |        679         318

- lrucache: loops only   eval: live

__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |      30893      28815 |       4198        2925
1  |  5          |      10491       7477 |       1527         664
2  |  25         |      14980       6836 |       1175         617
3  |  125        |      10749       6404 |        645         371
4  |  625        |      12659       6238 |        768         362
5  |  3125       |       8178       5731 |        664         338
6  |  15625      |       8204       5731 |        769         324
7  |  78125      |       8518       5410 |        649         314

- lrucache: tests only    eval: live

__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |      22389      20386 |       3766        2724
1  |  5          |        612        164 |        537         167
2  |  25         |        429        166 |        429         165
3  |  125        |        402        167 |        404         177
4  |  625        |        408        174 |        521         170
5  |  3125       |        449        164 |        421         176
6  |  15625      |        591        164 |        527         170
7  |  78125      |        497        163 |        680         165

- lrucache: 4   eval: live
__________________________________________________________________
i  |  n          | eval   avg        min | set    avg         min
------------------------------------------------------------------
0  |  1          |      44258      41357 |       6733        4782
1  |  5          |       4265        286 |       1113         284
2  |  25         |        539        186 |        474         187
3  |  125        |        442        185 |        436         183
4  |  625        |        448        184 |        491         165
5  |  3125       |        527        170 |        442         172
6  |  15625      |        798        163 |        483         164
7  |  78125      |        538        162 |        589         165


 * lrucache was used on either none, only loop functions, only test functions, or both test functions and loop functions.
"""



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" mathematicians.py - sample web scraping script
    (reference: https://realpython.com/python-web-scraping-practical-introduction/)

        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal
    """

if True:  # ! -- System Imports
    import sys
    from pprint import pprint
    from contextlib import closing

if True:  # ! -- Third Party Imports
    from requests import get
    from requests.exceptions import RequestException
    from bs4 import BeautifulSoup

    try:
        import lxml

        DEFAULT_PARSER = "lxml"
    except:
        DEFAULT_PARSER = "html.parser"

    # from autosys import *

# !---------------------------------------------- Common CONSTANTS
_debug_: bool = False
_fuzzy_: bool = True

DEFAULT_URL_HISTORY: int = 1000

# !---------------------------------------------- Custom Types


class ScrapeError(Exception):
    pass


def is_good_response(resp):
    """
    Returns True if the response seems to be HTML, False otherwise.
    """
    content_type = resp.headers["Content-Type"].lower()
    return (resp.status_code == 200 and content_type is not None
            and content_type.find("html") > -1)


def log_error(e):
    """
    It is always a good idea to log errors.
    This function just prints them, but you can
    make it do anything.
    """
    print(e)


def get_p_tags(html):
    for p in html.select("p"):
        yield p.text


def soup(url):
    return BeautifulSoup(simple_get(url), DEFAULT_PARSER)


# !---------------------------------------------- Script Tests
def __tests__(args) -> (int):
    """ Run Debug Tests for script if _debug_ = True. """

    print(f"{DEFAULT_PARSER=}")
    url = "https://realpython.com/blog/"
    raw_html = simple_get(url)
    html = soup(url)
    pprint(html)
    return 0


def __main__(args) -> (int):
    """ CLI script main entry point. """

    #! script testing
    if _debug_:
        __tests__(args)
    return 0


if __name__ == "__main__":  # if script is loaded directly from CLI
    __main__(sys.argv[1:])
""" Notes
    from tutorial ... moved out of __test__()

    raw_html = simple_get("https://realpython.com/blog/")
    print(len(raw_html))

    no_html = simple_get("https://realpython.com/blog/nope-not-gonna-find-it")

    print(no_html is None)

    # raw_html = open("contrived.html").read()
    """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" setup_utils Setup
    ---
    Part of the [AutoSys][1] package

    Copyright (c) 2018 [Michael Treanor][2]

    AutoSys is licensed under the [MIT License][3]

    [1]: https://www.github.com/skeptycal/autosys
    [2]: https://www.twitter.com/skeptycal
    [3]: https://opensource.org/licenses/MIT
    """

# Note: To use the 'upload' functionality of this file, you must:
#   $ pip install twine

from os import linesep as NL
from pathlib import Path
from sys import path as PYTHONPATH

from setuptools import find_packages, setup

from typing import Dict, Final, List, Optional, Sequence, Tuple

try:
    DEFAULT_ENCODING  #
except:
    from locale import getpreferredencoding
    DEFAULT_ENCODING: str = getpreferredencoding(do_setlocale=True) or "utf-8"

# the default version number is '0.0.1'
__version__: str = '0.0.1'

# put a name here to ignore the default ...
module_name: str = ""

# the default package name is the name of the parent folder ...
if not module_name:
    module_name = Path(__file__).resolve().parent.name

here = Path(__file__).resolve().parent
if here not in PYTHONPATH:
    PYTHONPATH.insert(0, here)


def table_print(data: (Dict, Sequence), **kwargs):
    tmp: List = []
    if isinstance(data, dict):
        tmp.extend(
            [f"{str(k):<15.15} :  {repr(v):<45.45}" for k, v in data.items()])
    elif isinstance(data, (list, tuple, set)):
        for x in data:
            try:
                tmp.append(f"{str(x):<15.15} :  {repr(f'{x}'):<45.45}")
            except:
                tmp.append(f"{str(x)}")
    else:
        raise TypeError('Parameter must be an iterable Mapping or Sequence.')
    print(NL.join(tmp), **kwargs)


def pip_safe_name(s: str):
    """ Return a name that is converted to pypi safe format. """
    return s.lower().replace("-", "_").replace(" ", "_")


def get_file_contents(file_name: str = "readme.md", search_list: List[str] = ["readme.md", "readme.rst", "readme", "readme.txt"]):
    """ Returns the text of the README file

        The default file is `README.md` and is *NOT* case sensitive. (e.g. `README` is the same as `readme`)
        Can load *any* text file, but the default search path is setup for readme files

        ```
        Search path = ["readme.md", "readme.rst", "readme", "readme.txt"]
        ```

        Example:

        ```
        long_description=readme()
        ```
        """

    if file_name not in search_list:
        search_list.insert(0, file_name)
    found: bool = False
    for searchfile in search_list:
        for parent in Path(__file__).resolve().parents:
            find_path = Path(parent / searchfile)
            if find_path.exists():
                found = True
                # print(find_path)
                break
        if found:
            break
    if found:
        try:
            with open(find_path, mode="r", encoding=DEFAULT_ENCODING) as f:
                return f.read()
        except IOError as e:
            raise IOError(f"Cannot read from the project file '{find_path}'")
    else:
        raise FileNotFoundError(
            f"Cannot find project file '{file_name}' in project tree. Search list = {search_list}"
        )


# ? **************************************** package metadata
# make the name safe for Pypi.org upload
NAME: str = pip_safe_name(module_name)

VERSION: str = __version__
VERSION_INFO: Tuple[int] = VERSION.split(".")
DESCRIPTION: str = "System utilities for Python on macOS."
REQUIRES_PYTHON: str = ">=3.8.0"
# PACKAGE_DIR: Dict = {f'{NAME}'}
PACKAGE_EXCLUDE: List[str] = ['*test*', '*bak*']
LICENSE: str = "MIT"
LONG_DESCRIPTION: str = get_file_contents('README.md')
LONG_DESCRIPTION_CONTENT_TYPE: str = "text/markdown"
# LONG_DESCRIPTION_CONTENT_TYPE="text/x-rst",
AUTHOR: str = "Michael Treanor"
AUTHOR_EMAIL: str = "skeptycal@gmail.com"
MAINTAINER: str = ""
MAINTAINER_EMAIL: str = ""
URL: str = f"https://skeptycal.github.io/{NAME}/"
DOWNLOAD_URL: str = f"https://github.com/skeptycal/{NAME}/archive/{VERSION}.tar.gz"
ZIP_SAFE: bool = False
INCLUDE_PACKAGE_DATA: bool = True
# What packages are required for this module to be executed?
REQUIRED: List[str] = [
    "aiocontextvars>=0.2.0 ; python_version<'3.7'",
    "colorama>=0.3.4 ; sys_platform=='win32'",
    "win32-setctime>=1.0.0 ; sys_platform=='win32'",
]

# What packages are optional?
EXTRAS: Dict = {}

PACKAGE_DATA: Dict = {
    # If any package contains these files, include them:
    "": [
        "*.txt",
        "*.rst",
        "*.md",
        "*.ini",
        "*.png",
        "*.jpg",
        "__init__.pyi",
        "py.typed",
    ]
}

PROJECT_URLS: Dict = {
    "Website": f"https://skeptycal.github.io/{NAME}/",
    "Documentation": f"https://skeptycal.github.io/{NAME}/docs",
    "Source Code": f"https://www.github.com/skeptycal/{NAME}/",
    "Changelog":
    f"https://github.com/skeptycal/{NAME}/blob/master/CHANGELOG.md",
}

KEYWORDS: List = [
    "application",
    "macOS",
    "dev",
    "devops",
    "cache",
    "utilities",
    "cli",
    "python",
    "cython",
    "text",
    "console",
    "log",
    "debug",
    "test",
    "testing",
    "logging",
    "logger",
]
CLASSIFIERS: List = [
    "Development Status :: 4 - Beta",
    "License :: OSI Approved :: MIT License",
    "Environment :: Console",
    "Environment :: MacOS X",
    "Intended Audience :: Developers",
    "Natural Language :: English",
    "Operating System :: MacOS",
    "Operating System :: OS Independent",
    "Programming Language :: Cython",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3 :: Only",
    "Programming Language :: Python :: 3",
    # These are the Python versions tested; it may work on others
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: Software Development :: Testing",
    "Topic :: Utilities",
]


def main():
    setup(
        name=NAME,
        version=VERSION,
        description=DESCRIPTION,
        python_requires=REQUIRES_PYTHON,
        # package_dir=PACKAGE_DIR,
        packages=find_packages(
            f'{NAME}', exclude=PACKAGE_EXCLUDE),
        # py_modules=[f"{NAME}"],
        license=LICENSE,
        long_description=LONG_DESCRIPTION,
        long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
        author=AUTHOR,
        author_email=AUTHOR_EMAIL,
        maintainer=MAINTAINER or AUTHOR,
        maintainer_email=MAINTAINER_EMAIL or AUTHOR_EMAIL,
        url=URL,
        download_url=DOWNLOAD_URL,
        zip_safe=ZIP_SAFE,
        include_package_data=INCLUDE_PACKAGE_DATA,
        # setup_requires=["isort"],
        install_requires=REQUIRED,
        extras_require=EXTRAS,
        package_data=PACKAGE_DATA,
        project_urls=PROJECT_URLS,
        keywords=KEYWORDS,
        classifiers=CLASSIFIERS,
    )


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# 'Standard Library'
# !---------------------------------------------- Utilities
import re
import sys

from pprint import pprint

# 'package imports'
from autosys.cli.debug import *
from autosys.defaults import _debug_

from typing import Dict, List

# regex
RE_SAFE_WORD_Pattern: re.Pattern = r"\w"
RE_NOT_SAFE_WORD_Pattern: re.Pattern = r"\W"
RE_NOT_ALPHA_UNDER_Pattern: re.Pattern = r"[^\w_]"

RE_SAFE_WORD = re.compile(fr"{RE_SAFE_WORD_Pattern}")
RE_NOT_SAFE_WORD = re.compile(fr"{RE_NOT_SAFE_WORD_Pattern}")

RE_NOT_ALPHA_UNDER = re.compile(fr"{RE_NOT_ALPHA_UNDER_Pattern}")

# pre-made translation table
TRANSLATE_TABLE_SAFE_WORD = dict.fromkeys(map(ord, "!@#$"), None)

# in place translation table creation
unicode_line: str = ""
unicode_line = unicode_line.translate({ord(c): None for c in "!@#$"})


def translate(translation_table, string):
    return string.translate(translation_table)


def safe_word(s):
    return re.sub(RE_NOT_SAFE_WORD, "", s)


def safe_filename(s):
    """ Truncates text and returns only alphanumeric and underscore characters. """
    s = fr"{s}"
    return re.sub(RE_NOT_ALPHA_UNDER, "", s)


class ASCIIRecord(str):
    """
    {
        name: str,
        code: int,
        char: str,
        desc: str
    }
    """

    pass
    # def __init__(self, name, num, desc):
    #     self._name = name
    #     self._num = num
    #     self._char = chr(num)
    #     self._desc = desc

    # def __str__(self):
    #     return self.char

    # @property
    # def name(self):
    #     return self._name

    # @property
    # def ord(self):
    #     return self._num

    # @property
    # def char(self):
    #     return self._char

    # @property
    # def desc(self):
    #     return self._desc

    # @property
    # def _unsafe(self):
    #     return self._char

    # @property
    # def utf8(self):
    #     return self._char.encode('')

    # @property
    # def info(self):
    #     return f'ASCII {self.name}'

    # def filesafe(self):
    #     " Return text with only alphanumeric and underscore characters."
    #     return self._char if re.match(r'\w', self._char) else ''


class ASCII:
    """
        # 10#13 — Indicates a new line &crarr  
        # 7 — bell (computer beeps)  
        # 26 — Ctrl+Z →  
        # 8 — backspace  
        # 127 — delete  
        # 27 — escape  
        # 32 — space  
        # 9 — tab  
        # 160 — a  
        # 163 — c  
        # 65 — A  
        # 68 — C  
        # 42 — *  
        """

    CRLF: str = chr(10) + chr(13)
    BELL: str = chr(7)
    BKSP: str = chr(8)
    TAB: str = chr(9)
    LF: str = chr(10)
    CTL_Z: str = chr(26)
    DEL: str = chr(127)
    ESC: str = chr(27)
    SPACE: str = chr(32)
    ASTERISK: str = chr(42)
    UPPER_A: str = chr(65)
    LOWER_A: str = chr(97)
    UPPER_C: str = chr(67)
    LOWER_C: str = chr(99)
    ELLIPSIS: str = chr(133)

    """ ASCII Control Codes:
        NUL (null)
        SOH (start of heading)
        STX (start of text)
        ETX (end of text)
        EOT (end of transmission) - Not the same as ETB
        ENQ (enquiry)
        ACK (acknowledge)
        BEL (bell) - Caused teletype machines to ring a bell. Causes a beep in many common terminals and terminal emulation programs.
        BS (backspace) - Moves the cursor (or print head) move backwards (left) one space.
        TAB (horizontal tab) - Moves the cursor (or print head) right to the next tab stop. The spacing of tab stops is dependent on the output device, but is often either 8 or 10.
        LF (NL line feed, new line) - Moves the cursor (or print head) to a new line. On Unix systems, moves to a new line AND all the way to the left.
        VT (vertical tab)
        FF (form feed) - Advances paper to the top of the next page (if the output device is a printer).
        CR (carriage return) - Moves the cursor all the way to the left, but does not advance to the next line.
        SO (shift out) - Switches output device to alternate character set.
        SI (shift in) - Switches output device back to default character set.
        DLE (data link escape)
        DC1 (device control 1)
        DC2 (device control 2)
        DC3 (device control 3)
        DC4 (device control 4)
        NAK (negative acknowledge)
        SYN (synchronous idle)
        ETB (end of transmission block) - Not the same as EOT
        CAN (cancel)
        EM (end of medium)
        SUB (substitute)
        ESC (escape)
        FS (file separator)
        GS (group separator)
        RS (record separator)
        US (unit separator)
        """


class Default:  # CLI Defaults
    """ Defaults for CLI formats and settings """

    SHOW_UNDERS = True
    SHOW_DUNDERS = True
    USE_REPR = False

    INDENT: int = 4
    TABLE_CHAR: str = "-"
    TABLE_GRAPHICS_TOP: str = ""
    TABLE_GRAPHICS_MID: str = ""
    TABLE_GRAPHICS_BOTTOM: str = ""

    DICT_KEY_WIDTH: int = 20
    DICT_VALUE_WIDTH: int = 35
    STR_WIDTH: int = 20
    LIST_ITEM_WIDTH: int = 35
    INT_WIDTH: float = 3.2

    FMT_DICT_KEY: str = f":>{DICT_KEY_WIDTH}"
    FMT_DICT_VALUE: str = f":>{DICT_VALUE_WIDTH}"
    FMT_LIST_ITEM_WIDTH: str = f":>{LIST_ITEM_WIDTH}"
    FMT_STR_WIDTH: str = f":>{STR_WIDTH}"
    FMT_INT_WIDTH: str = f":>{INT_WIDTH}"


class PrettyCLI:
    """ Provides a common cli interface. """

    def __init__(self, d: Dict):
        if not isinstance(d, dict):
            raise TypeError("Parameter should be of type `dict`")
        else:
            self._dict: Dict = d

    def print_dict(
        self,
        indent=Default.INDENT,
        group=10,
        blacklist=[],
        no_unders=True,
        no_dunders=True,
        value_repr=True,
        trim=True,
    ):
        c = Default.TABLE_CHAR
        kw = Default.DICT_KEY_WIDTH
        vw = Default.DICT_VALUE_WIDTH
        w = kw + vw + indent + 4
        TABLE_BREAK: str = c * w
        print(TABLE_BREAK)
        print(f"{'keys:':<{kw}s} -- {'values:':<{vw}}")
        for i, (k, v) in enumerate(self.items()):
            # k, v = x
            # kick out unwanted keys:
            if no_unders and k.startswith("_"):
                continue
            if no_dunders and k.startswith("__"):
                continue
            if k in blacklist:
                continue
            if int(i % group) == 0:
                print(TABLE_BREAK)

            if isinstance(v, dict):  # recursive if v is dict
                print_dict(v, indent=indent + Default.INDENT)
            else:
                v = f"{v!r}" if value_repr else f"{v!s}"
                v = str(v)
                if trim:
                    try:
                        v = v[: vw - 1] + ASCII.ELLIPSIS
                    except:
                        pass
                    try:
                        k = k[: kw - 1]
                    except:
                        pass
                    # v = f"{v:<{vw}}"
                print(f"{' '*indent}{k:<{kw}s} -- {v:<{vw}s}")
        print(TABLE_BREAK)
        return 0

    def print_list(
        self,
        indent=Default.INDENT,
        group=10,
        blacklist=[],
        no_unders=True,
        no_dunders=True,
        value_repr=True,
        trim=True,
    ):
        c = Default.TABLE_CHAR
        kw = Default.DICT_KEY_WIDTH
        vw = Default.DICT_VALUE_WIDTH
        w = kw + vw + indent + 4
        TABLE_BREAK: str = c * w
        print(TABLE_BREAK)
        print(f"{'keys:':<{kw}s} -- {'values:':<{vw}}")
        for i, (k, v) in enumerate(self.items()):
            # k, v = x
            # kick out unwanted keys:
            if no_unders and k.startswith("_"):
                continue
            if no_dunders and k.startswith("__"):
                continue
            if k in blacklist:
                continue
            if int(i % group) == 0:
                print(TABLE_BREAK)

            if isinstance(v, dict):  # recursive if v is dict
                print_dict(v, indent=indent + Default.INDENT)
            else:
                v = f"{v!r}" if value_repr else f"{v!s}"
                v = str(v)
                if trim:
                    try:
                        v = v[: vw - 1] + ASCII.ELLIPSIS
                    except:
                        pass
                    try:
                        k = k[: kw - 1]
                    except:
                        pass
                    # v = f"{v:<{vw}}"
                print(f"{' '*indent}{k:<{kw}s} -- {v:<{vw}s}")
        print(TABLE_BREAK)
        return 0


if True:  # !------------------------ Script Tests

    def _tests_(args):
        """
        Run Debug Tests for script if _debug_ = True.
        """
        print(f"{args=}")
        print()
        print(get_class_name("platform.uname()"))
        print(get_class_name("LOG_PATH"))
        print(get_class_name("SCRIPT_PATH()"))
        print(get_class_name("_debug_"))
        return 0

    def _main_(args):
        """ CLI script main entry point. """

        #! script testing
        if _debug_:
            # inject `args` here ...
            args.append("--version")
            args.append("--verbose")
            return _tests_(args)
        return 0


if __name__ == "__main__":  # if script is loaded directly from CLI
    _main_(ARGS())
test_str = r"f_fkj3*$//\\fadkjkk"
print(RE_NOT_ALPHA_UNDER)
print(safe_word(test_str))
print(safe_filename(test_str))
# pprint(dir(), indent=4, width=50, depth=3)



# pylint: disable=invalid-name
# 'package imports'
import directories
import files

temp_file = temporary.files.temp_file
temp_dir = temporary.directories.temp_dir
in_temp_dir = temporary.directories.in_temp_dir



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" # TODO -- @update `AutoSys` package
        copyright (c) 2018 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """

_blacklist = [
    "Field",
    "Final",
    "List",
    "arepl_store",
    "choice",
    "get_all",
    "help",
    "howdoi",
    "input",
    "linesep",
    "show_all",
    "Tuple",
    "dataclass",
    "field",
]


def show_all(x):
    print("__all__: List[str] = [")
    for i in x:
        if i.startswith("_"):
            continue
        if i in _blacklist:
            continue
        print(f"    '{i}',")
    print("]")



# Python Tools for Visual Studio
# Copyright(c) Microsoft Corporation
# All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the License); you may not use
# this file except in compliance with the License. You may obtain a copy of the
# License at http://www.apache.org/licenses/LICENSE-2.0
#
# THIS CODE IS PROVIDED ON AN  *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS
# OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY
# IMPLIED WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
# MERCHANTABILITY OR NON-INFRINGEMENT.
#
# See the Apache Version 2.0 License for specific language governing
# permissions and limitations under the License.

# Supported Python versions: 2.7, 3.5+
# https://devguide.python.org/#status-of-python-branches

# 'future imports'
from __future__ import print_function

# 'Standard Library'
import importlib
import inspect
import json
import os.path
import sys
import time

# Uncomment to send stderr somewhere readable.
# sys.stderr = open(os.path.join(os.path.expanduser("~"), "log.txt"), "a")

if sys.version_info >= (3,):
    stdout = sys.stdout.buffer
    stdin = sys.stdin.buffer
else:
    stdout = sys.stdout
    stdin = sys.stdin

    if sys.platform == "win32":
        import os, msvcrt

        msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
        msvcrt.setmode(sys.stdin.fileno(), os.O_BINARY)


def read_line():
    line = b""
    while True:
        try:
            line += stdin.readline()
        except:
            raise EOFError
        if not line:
            raise EOFError
        if line.endswith(b"\r\n"):
            return line[:-2]


def write_stdout(data):
    if not isinstance(data, bytes):
        data = data.encode("utf-8")

    stdout.write(data)
    stdout.flush()


METHOD_NOT_FOUND = -32601
INTERNAL_ERROR = -32603


def read_request():
    headers = dict()

    while True:
        line = read_line()

        if line == b"":
            break

        key, _, value = line.partition(b": ")
        headers[key] = value

    length = int(headers[b"Content-Length"])

    body = b""
    while length > 0:
        chunk = stdin.read(length)
        body += chunk
        length -= len(chunk)

    request = json.loads(body)
    return Request(request)


def requests():
    while True:
        try:
            request = read_request()
        except EOFError:
            return

        yield request


def write_response(id, d):
    response = {"jsonrpc": "2.0", "id": id}
    response.update(d)

    s = json.dumps(response)

    data = "Content-Length: {}\r\n\r\n".format(len(s)) + s
    write_stdout(data)


class Request(object):
    def __init__(self, request):
        self.id = request["id"]
        self.method = request["method"]
        self.params = request.get("params", None)

    def write_result(self, result):
        write_response(self.id, {"result": result})

    def write_error(self, code, message):
        write_response(self.id, {"error": {"code": code, "message": message}})


class Mux(object):
    def __init__(self):
        self.handlers = {}

    def handler(self, method):
        def decorator(func):
            self.handlers[method] = func
            return func

        return decorator

    def handle(self, request):
        handler = self.handlers.get(request.method, None)

        if not handler:
            request.write_error(
                METHOD_NOT_FOUND, "method {} not found".format(request.method)
            )
            return

        try:
            result = handler(*request.params)
        except Exception as e:
            request.write_error(INTERNAL_ERROR, str(e))
        else:
            request.write_result(result)


mux = Mux()


def do_not_inspect(v):
    # https://github.com/Microsoft/python-language-server/issues/740
    # https://github.com/cython/cython/issues/1470
    if type(v).__name__ != "fused_cython_function":
        return False

    # If a fused function has __defaults__, then attempting to access
    # __kwdefaults__ will fail if generated before cython 0.29.6.
    return bool(getattr(v, "__defaults__", False))


KNOWN_DIST_PREFIXES = {"PyQt5": ["PyQt5"], "PyQt5-sip": ["PyQt5.sip"]}


def build_dist_prefixes():
    import pkg_resources

    prefixes = dict()

    # This iterates in the order things were added; no need to reverse.
    for d in pkg_resources.WorkingSet():
        top_level = None

        try:
            top_level = d.get_metadata("top_level.txt")
        except:
            pass

        module_prefixes = None
        if top_level:
            module_prefixes = top_level.splitlines()
        elif d.project_name:
            module_prefixes = KNOWN_DIST_PREFIXES.get(d.project_name, None)

        if not module_prefixes:
            continue

        for prefix in module_prefixes:
            prefix = prefix.strip()
            if prefix:
                prefixes[prefix] = d

    return prefixes


DIST_PREFIXES = None


def find_dist(module_name):
    global DIST_PREFIXES
    if DIST_PREFIXES is None:
        DIST_PREFIXES = build_dist_prefixes()

    while True:
        if not module_name:
            return None

        d = DIST_PREFIXES.get(module_name, None)
        if d:
            return d

        split = module_name.split(".")
        if len(split) < 2:
            return None

        module_name = ".".join(split[:-1])


@mux.handler("$/cancelRequest")
def cancel_request(params):
    return None


@mux.handler("moduleMemberNames")
def module_member_names(module_name):
    try:
        module = importlib.import_module(module_name)
    except:
        return None

    members = inspect.getmembers(module)

    return {
        "members": [name for name, _ in members],
        "all": getattr(module, "__all__", None),
    }


@mux.handler("moduleVersion")
def module_version(module_name):
    version = None
    try:
        # TODO: iterate as in find_dist and import looking for __version__?
        module = importlib.import_module(module_name)
        version = getattr(module, "__version__", None)
    except:
        pass

    if not version:
        try:
            d = find_dist(module_name)
            if d:
                version = d.version
        except:
            pass

    return version


def main():
    for request in requests():
        mux.handle(request)


if __name__ == "__main__":
    main()



#!/usr/bin/env python

# Copyright 2016 The Python-Twitter Developers
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# ------------------------------------------------------------------------
# Change History
# 2010-10-01
#   Initial commit by @jsteiner207
#
# 2014-12-29
#   PEP8 update by @radzhome
#
# 2016-05-07
#   Update for Python3 by @jeremylow
#

# 'future imports'
from __future__ import print_function

# 'package imports'
import twitter

CONSUMER_KEY = "WWWWWWWW"
CONSUMER_SECRET = "XXXXXXXX"
ACCESS_TOKEN = "YYYYYYYY"
ACCESS_TOKEN_SECRET = "ZZZZZZZZ"


# Create an Api instance.
api = twitter.Api(
    consumer_key=CONSUMER_KEY,
    consumer_secret=CONSUMER_SECRET,
    access_token_key=ACCESS_TOKEN,
    access_token_secret=ACCESS_TOKEN_SECRET,
)

users = api.GetFriends()

print([u.screen_name for u in users])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" Add parentheses around print arguments.

    Used in translating Python 2 code modules to Python 3.

    https://pymotw.com/2/traceback/index.html#module-traceback
    The examples below use the module traceback_example.py (provided in the source package for PyMOTW). The contents are:
    """

# 'Standard Library'
import re
import sys
import traceback

"""
Add parens around print arguments
VS Code regex replacement for print function:
    print ([']{1}.*)\n
    print($1)\n
"""

FIND_PARENTS: re.Pattern = re.compile(r"([']{1}.*)\n")


def fix_parens(filenames):
    for file in filenames:
        with open(file, mode="r") as f:
            for line in f.readlines():
                fp = re.Pattern.sub()


if __name__ == "__main__":
    _debug_: bool = True  # True => use Debug features
    _verbose_: int = 0  # Verbosity level => 0 - 4
    _log_flag_: bool = _debug_ and True  # True => log to file if _debug_
    # argv.append('version')  # ! test
    # argv.append('help')  # ! test
    fix_parens(sys.argv[1:])



# -*- coding: utf-8 -*-
"""Collection of utilities for use in API calls, functions."""
# 'future imports'
from __future__ import unicode_literals

# 'Standard Library'
import mimetypes
import os
import re
import sys

from tempfile import NamedTemporaryFile
from unicodedata import normalize

# 'package imports'
import twitter

from twitter import TwitterError

# 'third party'
import requests

try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse


if sys.version_info < (3,):
    range = xrange  # noqa

if sys.version_info > (3,):
    unicode = str

CHAR_RANGES = [
    range(0, 4351),
    range(8192, 8205),
    range(8208, 8223),
    range(8242, 8247),
]

TLDS = [
    "ac",
    "ad",
    "ae",
    "af",
    "ag",
    "ai",
    "al",
    "am",
    "an",
    "ao",
    "aq",
    "ar",
    "as",
    "at",
    "au",
    "aw",
    "ax",
    "az",
    "ba",
    "bb",
    "bd",
    "be",
    "bf",
    "bg",
    "bh",
    "bi",
    "bj",
    "bl",
    "bm",
    "bn",
    "bo",
    "bq",
    "br",
    "bs",
    "bt",
    "bv",
    "bw",
    "by",
    "bz",
    "ca",
    "cc",
    "cd",
    "cf",
    "cg",
    "ch",
    "ci",
    "ck",
    "cl",
    "cm",
    "cn",
    "co",
    "cr",
    "cu",
    "cv",
    "cw",
    "cx",
    "cy",
    "cz",
    "de",
    "dj",
    "dk",
    "dm",
    "do",
    "dz",
    "ec",
    "ee",
    "eg",
    "eh",
    "er",
    "es",
    "et",
    "eu",
    "fi",
    "fj",
    "fk",
    "fm",
    "fo",
    "fr",
    "ga",
    "gb",
    "gd",
    "ge",
    "gf",
    "gg",
    "gh",
    "gi",
    "gl",
    "gm",
    "gn",
    "gp",
    "gq",
    "gr",
    "gs",
    "gt",
    "gu",
    "gw",
    "gy",
    "hk",
    "hm",
    "hn",
    "hr",
    "ht",
    "hu",
    "id",
    "ie",
    "il",
    "im",
    "in",
    "io",
    "iq",
    "ir",
    "is",
    "it",
    "je",
    "jm",
    "jo",
    "jp",
    "ke",
    "kg",
    "kh",
    "ki",
    "km",
    "kn",
    "kp",
    "kr",
    "kw",
    "ky",
    "kz",
    "la",
    "lb",
    "lc",
    "li",
    "lk",
    "lr",
    "ls",
    "lt",
    "lu",
    "lv",
    "ly",
    "ma",
    "mc",
    "md",
    "me",
    "mf",
    "mg",
    "mh",
    "mk",
    "ml",
    "mm",
    "mn",
    "mo",
    "mp",
    "mq",
    "mr",
    "ms",
    "mt",
    "mu",
    "mv",
    "mw",
    "mx",
    "my",
    "mz",
    "na",
    "nc",
    "ne",
    "nf",
    "ng",
    "ni",
    "nl",
    "no",
    "np",
    "nr",
    "nu",
    "nz",
    "om",
    "pa",
    "pe",
    "pf",
    "pg",
    "ph",
    "pk",
    "pl",
    "pm",
    "pn",
    "pr",
    "ps",
    "pt",
    "pw",
    "py",
    "qa",
    "re",
    "ro",
    "rs",
    "ru",
    "rw",
    "sa",
    "sb",
    "sc",
    "sd",
    "se",
    "sg",
    "sh",
    "si",
    "sj",
    "sk",
    "sl",
    "sm",
    "sn",
    "so",
    "sr",
    "ss",
    "st",
    "su",
    "sv",
    "sx",
    "sy",
    "sz",
    "tc",
    "td",
    "tf",
    "tg",
    "th",
    "tj",
    "tk",
    "tl",
    "tm",
    "tn",
    "to",
    "tp",
    "tr",
    "tt",
    "tv",
    "tw",
    "tz",
    "ua",
    "ug",
    "uk",
    "um",
    "us",
    "uy",
    "uz",
    "va",
    "vc",
    "ve",
    "vg",
    "vi",
    "vn",
    "vu",
    "wf",
    "ws",
    "ye",
    "yt",
    "za",
    "zm",
    "zw",
    "ελ",
    "бел",
    "мкд",
    "мон",
    "рф",
    "срб",
    "укр",
    "қаз",
    "հայ",
    "الاردن",
    "الجزائر",
    "السعودية",
    "المغرب",
    "امارات",
    "ایران",
    "بھارت",
    "تونس",
    "سودان",
    "سورية",
    "عراق",
    "عمان",
    "فلسطين",
    "قطر",
    "مصر",
    "مليسيا",
    "پاکستان",
    "भारत",
    "বাংলা",
    "ভারত",
    "ਭਾਰਤ",
    "ભારત",
    "இந்தியா",
    "இலங்கை",
    "சிங்கப்பூர்",
    "భారత్",
    "ලංකා",
    "ไทย",
    "გე",
    "中国",
    "中國",
    "台湾",
    "台灣",
    "新加坡",
    "澳門",
    "香港",
    "한국",
    "neric:",
    "abb",
    "abbott",
    "abogado",
    "academy",
    "accenture",
    "accountant",
    "accountants",
    "aco",
    "active",
    "actor",
    "ads",
    "adult",
    "aeg",
    "aero",
    "afl",
    "agency",
    "aig",
    "airforce",
    "airtel",
    "allfinanz",
    "alsace",
    "amsterdam",
    "android",
    "apartments",
    "app",
    "aquarelle",
    "archi",
    "army",
    "arpa",
    "asia",
    "associates",
    "attorney",
    "auction",
    "audio",
    "auto",
    "autos",
    "axa",
    "azure",
    "band",
    "bank",
    "bar",
    "barcelona",
    "barclaycard",
    "barclays",
    "bargains",
    "bauhaus",
    "bayern",
    "bbc",
    "bbva",
    "bcn",
    "beer",
    "bentley",
    "berlin",
    "best",
    "bet",
    "bharti",
    "bible",
    "bid",
    "bike",
    "bing",
    "bingo",
    "bio",
    "biz",
    "black",
    "blackfriday",
    "bloomberg",
    "blue",
    "bmw",
    "bnl",
    "bnpparibas",
    "boats",
    "bond",
    "boo",
    "boots",
    "boutique",
    "bradesco",
    "bridgestone",
    "broker",
    "brother",
    "brussels",
    "budapest",
    "build",
    "builders",
    "business",
    "buzz",
    "bzh",
    "cab",
    "cafe",
    "cal",
    "camera",
    "camp",
    "cancerresearch",
    "canon",
    "capetown",
    "capital",
    "caravan",
    "cards",
    "care",
    "career",
    "careers",
    "cars",
    "cartier",
    "casa",
    "cash",
    "casino",
    "cat",
    "catering",
    "cba",
    "cbn",
    "ceb",
    "center",
    "ceo",
    "cern",
    "cfa",
    "cfd",
    "chanel",
    "channel",
    "chat",
    "cheap",
    "chloe",
    "christmas",
    "chrome",
    "church",
    "cisco",
    "citic",
    "city",
    "claims",
    "cleaning",
    "click",
    "clinic",
    "clothing",
    "cloud",
    "club",
    "coach",
    "codes",
    "coffee",
    "college",
    "cologne",
    "com",
    "commbank",
    "community",
    "company",
    "computer",
    "condos",
    "construction",
    "consulting",
    "contractors",
    "cooking",
    "cool",
    "coop",
    "corsica",
    "country",
    "coupons",
    "courses",
    "credit",
    "creditcard",
    "cricket",
    "crown",
    "crs",
    "cruises",
    "cuisinella",
    "cymru",
    "cyou",
    "dabur",
    "dad",
    "dance",
    "date",
    "dating",
    "datsun",
    "day",
    "dclk",
    "deals",
    "degree",
    "delivery",
    "delta",
    "democrat",
    "dental",
    "dentist",
    "desi",
    "design",
    "dev",
    "diamonds",
    "diet",
    "digital",
    "direct",
    "directory",
    "discount",
    "dnp",
    "docs",
    "dog",
    "doha",
    "domains",
    "doosan",
    "download",
    "drive",
    "durban",
    "dvag",
    "earth",
    "eat",
    "edu",
    "education",
    "email",
    "emerck",
    "energy",
    "engineer",
    "engineering",
    "enterprises",
    "epson",
    "equipment",
    "erni",
    "esq",
    "estate",
    "eurovision",
    "eus",
    "events",
    "everbank",
    "exchange",
    "expert",
    "exposed",
    "express",
    "fage",
    "fail",
    "faith",
    "family",
    "fan",
    "fans",
    "farm",
    "fashion",
    "feedback",
    "film",
    "finance",
    "financial",
    "firmdale",
    "fish",
    "fishing",
    "fit",
    "fitness",
    "flights",
    "florist",
    "flowers",
    "flsmidth",
    "fly",
    "foo",
    "football",
    "forex",
    "forsale",
    "forum",
    "foundation",
    "frl",
    "frogans",
    "fund",
    "furniture",
    "futbol",
    "fyi",
    "gal",
    "gallery",
    "game",
    "garden",
    "gbiz",
    "gdn",
    "gent",
    "genting",
    "ggee",
    "gift",
    "gifts",
    "gives",
    "giving",
    "glass",
    "gle",
    "global",
    "globo",
    "gmail",
    "gmo",
    "gmx",
    "gold",
    "goldpoint",
    "golf",
    "goo",
    "goog",
    "google",
    "gop",
    "gov",
    "graphics",
    "gratis",
    "green",
    "gripe",
    "group",
    "guge",
    "guide",
    "guitars",
    "guru",
    "hamburg",
    "hangout",
    "haus",
    "healthcare",
    "help",
    "here",
    "hermes",
    "hiphop",
    "hitachi",
    "hiv",
    "hockey",
    "holdings",
    "holiday",
    "homedepot",
    "homes",
    "honda",
    "horse",
    "host",
    "hosting",
    "hoteles",
    "hotmail",
    "house",
    "how",
    "hsbc",
    "ibm",
    "icbc",
    "ice",
    "icu",
    "ifm",
    "iinet",
    "immo",
    "immobilien",
    "industries",
    "infiniti",
    "info",
    "ing",
    "ink",
    "institute",
    "insure",
    "int",
    "international",
    "investments",
    "ipiranga",
    "irish",
    "ist",
    "istanbul",
    "itau",
    "iwc",
    "java",
    "jcb",
    "jetzt",
    "jewelry",
    "jlc",
    "jll",
    "jobs",
    "joburg",
    "jprs",
    "juegos",
    "kaufen",
    "kddi",
    "kim",
    "kitchen",
    "kiwi",
    "koeln",
    "komatsu",
    "krd",
    "kred",
    "kyoto",
    "lacaixa",
    "lancaster",
    "land",
    "lasalle",
    "lat",
    "latrobe",
    "law",
    "lawyer",
    "lds",
    "lease",
    "leclerc",
    "legal",
    "lexus",
    "lgbt",
    "liaison",
    "lidl",
    "life",
    "lighting",
    "limited",
    "limo",
    "link",
    "live",
    "lixil",
    "loan",
    "loans",
    "lol",
    "london",
    "lotte",
    "lotto",
    "love",
    "ltda",
    "lupin",
    "luxe",
    "luxury",
    "madrid",
    "maif",
    "maison",
    "man",
    "management",
    "mango",
    "market",
    "marketing",
    "markets",
    "marriott",
    "mba",
    "media",
    "meet",
    "melbourne",
    "meme",
    "memorial",
    "men",
    "menu",
    "miami",
    "microsoft",
    "mil",
    "mini",
    "mma",
    "mobi",
    "moda",
    "moe",
    "mom",
    "monash",
    "money",
    "montblanc",
    "mormon",
    "mortgage",
    "moscow",
    "motorcycles",
    "mov",
    "movie",
    "movistar",
    "mtn",
    "mtpc",
    "museum",
    "nadex",
    "nagoya",
    "name",
    "navy",
    "nec",
    "net",
    "netbank",
    "network",
    "neustar",
    "new",
    "news",
    "nexus",
    "ngo",
    "nhk",
    "nico",
    "ninja",
    "nissan",
    "nokia",
    "nra",
    "nrw",
    "ntt",
    "nyc",
    "office",
    "okinawa",
    "omega",
    "one",
    "ong",
    "onl",
    "online",
    "ooo",
    "oracle",
    "orange",
    "org",
    "organic",
    "osaka",
    "otsuka",
    "ovh",
    "page",
    "panerai",
    "paris",
    "partners",
    "parts",
    "party",
    "pet",
    "pharmacy",
    "philips",
    "photo",
    "photography",
    "photos",
    "physio",
    "piaget",
    "pics",
    "pictet",
    "pictures",
    "pink",
    "pizza",
    "place",
    "play",
    "plumbing",
    "plus",
    "pohl",
    "poker",
    "porn",
    "post",
    "praxi",
    "press",
    "pro",
    "prod",
    "productions",
    "prof",
    "properties",
    "property",
    "pub",
    "qpon",
    "quebec",
    "racing",
    "realtor",
    "realty",
    "recipes",
    "red",
    "redstone",
    "rehab",
    "reise",
    "reisen",
    "reit",
    "ren",
    "rent",
    "rentals",
    "repair",
    "report",
    "republican",
    "rest",
    "restaurant",
    "review",
    "reviews",
    "rich",
    "ricoh",
    "rio",
    "rip",
    "rocks",
    "rodeo",
    "rsvp",
    "ruhr",
    "run",
    "ryukyu",
    "saarland",
    "sakura",
    "sale",
    "samsung",
    "sandvik",
    "sandvikcoromant",
    "sanofi",
    "sap",
    "sarl",
    "saxo",
    "sca",
    "scb",
    "schmidt",
    "scholarships",
    "school",
    "schule",
    "schwarz",
    "science",
    "scor",
    "scot",
    "seat",
    "seek",
    "sener",
    "services",
    "sew",
    "sex",
    "sexy",
    "shiksha",
    "shoes",
    "show",
    "shriram",
    "singles",
    "site",
    "ski",
    "sky",
    "skype",
    "sncf",
    "soccer",
    "social",
    "software",
    "sohu",
    "solar",
    "solutions",
    "sony",
    "soy",
    "space",
    "spiegel",
    "spreadbetting",
    "srl",
    "starhub",
    "statoil",
    "studio",
    "study",
    "style",
    "sucks",
    "supplies",
    "supply",
    "support",
    "surf",
    "surgery",
    "suzuki",
    "swatch",
    "swiss",
    "sydney",
    "systems",
    "taipei",
    "tatamotors",
    "tatar",
    "tattoo",
    "tax",
    "taxi",
    "team",
    "tech",
    "technology",
    "tel",
    "telefonica",
    "temasek",
    "tennis",
    "thd",
    "theater",
    "tickets",
    "tienda",
    "tips",
    "tires",
    "tirol",
    "today",
    "tokyo",
    "tools",
    "top",
    "toray",
    "toshiba",
    "tours",
    "town",
    "toyota",
    "toys",
    "trade",
    "trading",
    "training",
    "travel",
    "trust",
    "tui",
    "ubs",
    "university",
    "uno",
    "uol",
    "vacations",
    "vegas",
    "ventures",
    "vermögensberater",
    "vermögensberatung",
    "versicherung",
    "vet",
    "viajes",
    "video",
    "villas",
    "vin",
    "vision",
    "vista",
    "vistaprint",
    "vlaanderen",
    "vodka",
    "vote",
    "voting",
    "voto",
    "voyage",
    "wales",
    "walter",
    "wang",
    "watch",
    "webcam",
    "website",
    "wed",
    "wedding",
    "weir",
    "whoswho",
    "wien",
    "wiki",
    "williamhill",
    "win",
    "windows",
    "wine",
    "wme",
    "work",
    "works",
    "world",
    "wtc",
    "wtf",
    "xbox",
    "xerox",
    "xin",
    "xperia",
    "xxx",
    "xyz",
    "yachts",
    "yandex",
    "yodobashi",
    "yoga",
    "yokohama",
    "youtube",
    "zip",
    "zone",
    "zuerich",
    "дети",
    "ком",
    "москва",
    "онлайн",
    "орг",
    "рус",
    "сайт",
    "קום",
    "بازار",
    "شبكة",
    "كوم",
    "موقع",
    "कॉम",
    "नेट",
    "संगठन",
    "คอม",
    "みんな",
    "グーグル",
    "コム",
    "世界",
    "中信",
    "中文网",
    "企业",
    "佛山",
    "信息",
    "健康",
    "八卦",
    "公司",
    "公益",
    "商城",
    "商店",
    "商标",
    "在线",
    "大拿",
    "娱乐",
    "工行",
    "广东",
    "慈善",
    "我爱你",
    "手机",
    "政务",
    "政府",
    "新闻",
    "时尚",
    "机构",
    "淡马锡",
    "游戏",
    "点看",
    "移动",
    "组织机构",
    "网址",
    "网店",
    "网络",
    "谷歌",
    "集团",
    "飞利浦",
    "餐厅",
    "닷넷",
    "닷컴",
    "삼성",
    "onion",
]

URL_REGEXP = re.compile(
    (
        r"("
        r"^(?!(https?://|www\.)?\.|ftps?://|([0-9]+\.){{1,3}}\d+)"  # exclude urls that start with "."
        r"(?:https?://|www\.)*^(?!.*@)(?:[\w+-_]+[.])"  # beginning of url
        r"(?:{0}\b"  # all tlds
        r"(?:[:0-9]))"  # port numbers & close off TLDs
        r"(?:[\w+\/]?[a-z0-9!\*\'\(\);:&=\+\$/%#\[\]\-_\.,~?])*"  # path/query params
        r")"
    ).format(r"\b|".join(TLDS)),
    re.U | re.I | re.X,
)


def calc_expected_status_length(status, short_url_length=23):
    """Calculate the length of a tweet.

    Takes into account Twitter's replacement of URLs with https://t.co links.

    Args:
        status: text of the status message to be posted.
        short_url_length: the current published https://t.co links

    Returns:
        Expected length of the status message as an integer.
    """
    status_length = 0
    if isinstance(status, bytes):
        status = unicode(status)
    for word in re.split(r"\s", status):
        if is_url(word):
            status_length += short_url_length
        else:
            for character in word:
                if any(
                    [
                        ord(normalize("NFC", character)) in char_range
                        for char_range in CHAR_RANGES
                    ]
                ):
                    status_length += 1
                else:
                    status_length += 2
    status_length += len(re.findall(r"\s", status))
    return status_length


def is_url(text):
    """Check to see if a bit of text is a URL.

    Args:
        text: text to check.

    Returns:
        Boolean of whether the text should be treated as a URL or not.
    """
    return bool(re.findall(URL_REGEXP, text))


def http_to_file(http):
    """Turn a URL into a file-like object.

    Args:
        http (str): URL of the file to download.

    Returns:
        File-like object of downloaded URL.
    """
    data_file = NamedTemporaryFile()
    req = requests.get(http, stream=True)
    for chunk in req.iter_content(chunk_size=1024 * 1024):
        data_file.write(chunk)
    return data_file


def parse_media_file(passed_media, async_upload=False):
    """Parse a media file and attempts to return a file-like object and information about the media file.

    Args:
        passed_media: media file which to parse.
        async_upload: flag, for validation media file attributes.

    Returns:
        file-like object, the filename of the media file, the file size, and
        the type of media.
    """
    img_formats = ["image/jpeg", "image/png", "image/bmp", "image/webp"]
    long_img_formats = ["image/gif"]
    subtitle_formats = ["text/srt"]
    video_formats = ["video/mp4", "video/quicktime"]

    # If passed_media is a string, check if it points to a URL, otherwise,
    # it should point to local file. Create a reference to a file obj for
    #  each case such that data_file ends up with a read() method.
    if not hasattr(passed_media, "read"):
        if passed_media.startswith("http"):
            data_file = http_to_file(passed_media)
            filename = os.path.basename(urlparse(passed_media).path)
        else:
            data_file = open(os.path.realpath(passed_media), "rb")
            filename = os.path.basename(passed_media)

    # Otherwise, if a file object was passed in the first place,
    # create the standard reference to media_file (i.e., rename it to fp).
    else:
        if passed_media.mode not in ["rb", "rb+", "w+b"]:
            raise TwitterError('File mode must be "rb" or "rb+"')
        filename = os.path.basename(passed_media.name)
        data_file = passed_media

    data_file.seek(0, 2)
    file_size = data_file.tell()

    try:
        data_file.seek(0)
    except Exception as e:
        pass

    media_type = mimetypes.guess_type(os.path.basename(filename))[0]
    # The .srt extension is not recognised by the mimetypes module.
    if os.path.basename(filename).endswith(".srt"):
        media_type = "text/srt"
    if media_type is not None:
        if media_type in img_formats and file_size > 5 * 1048576:
            raise TwitterError({"message": "Images must be less than 5MB."})
        elif media_type in long_img_formats and file_size > 15 * 1048576:
            raise TwitterError(
                {"message": "GIF Image must be less than 15MB."}
            )
        elif (
            media_type in video_formats
            and not async_upload
            and file_size > 15 * 1048576
        ):
            raise TwitterError({"message": "Videos must be less than 15MB."})
        elif (
            media_type in video_formats
            and async_upload
            and file_size > 512 * 1048576
        ):
            raise TwitterError({"message": "Videos must be less than 512MB."})
        elif (
            media_type
            not in img_formats
            + long_img_formats
            + subtitle_formats
            + video_formats
        ):
            raise TwitterError(
                {"message": "Media type could not be determined."}
            )

    return data_file, filename, file_size, media_type


def enf_type(field, _type, val):
    """Enforce type checking on variable.

    Check to see if a given val for a field (i.e., the name
    of the field) is of the proper _type. If it is not,
    raise a TwitterError with a brief explanation.

    Args:
        field:
            Name of the field you are checking.
        _type:
            Type that the value should be returned as.
        val:
            Value to convert to _type.

    Returns:
        val converted to type _type.

    """
    try:
        return _type(val)
    except ValueError:
        raise TwitterError(
            {"message": '"{0}" must be type {1}'.format(field, _type.__name__)}
        )


def parse_arg_list(args, attr):
    """Parse a set of args for list functions.

    Convenience/DRY function.

    Args:
        args (str, twitter.User, list, tuple): set of arguments to
            pass to API.
        attr (str): The attribute to be extracted from each arg.

    Returns:
        (str) A string representing the concatenated arguments.
    """
    out = []
    if isinstance(args, (str, unicode)):
        out.append(args)
    elif isinstance(args, twitter.User):
        out.append(getattr(args, attr))
    elif isinstance(args, (list, tuple)):
        for item in args:
            if isinstance(item, (str, unicode)):
                out.append(item)
            elif isinstance(item, twitter.User):
                out.append(getattr(item, attr))
    return ",".join([str(item) for item in out])



#!/usr/bin/env python

"""Post a message to twitter"""

__author__ = "dewitt@google.com"

from __future__ import print_function

import getopt
import os
import sys

import twitter

try:
    import configparser
except ImportError as _:
    import ConfigParser as configparser



USAGE = """Usage: tweet [options] message

  This script posts a message to Twitter.

  Options:

    -h --help : print this help
    --consumer-key : the twitter consumer key
    --consumer-secret : the twitter consumer secret
    --access-key : the twitter access token key
    --access-secret : the twitter access token secret
    --encoding : the character set encoding used in input strings, e.g. "utf-8". [optional]

  Documentation:

  If either of the command line flags are not present, the environment
  variables TWEETUSERNAME and TWEETPASSWORD will then be checked for your
  consumer_key or consumer_secret, respectively.

  If neither the command line flags nor the environment variables are
  present, the .tweetrc file, if it exists, can be used to set the
  default consumer_key and consumer_secret.  The file should contain the
  following three lines, replacing *consumer_key* with your consumer key, and
  *consumer_secret* with your consumer secret:

  A skeletal .tweetrc file:

    [Tweet]
    consumer_key: *consumer_key*
    consumer_secret: *consumer_password*
    access_key: *access_key*
    access_secret: *access_password*

"""


def PrintUsageAndExit():
    print(USAGE)
    sys.exit(2)


def GetConsumerKeyEnv():
    return os.environ.get("TWEETUSERNAME", None)


def GetConsumerSecretEnv():
    return os.environ.get("TWEETPASSWORD", None)


def GetAccessKeyEnv():
    return os.environ.get("TWEETACCESSKEY", None)


def GetAccessSecretEnv():
    return os.environ.get("TWEETACCESSSECRET", None)


class TweetRc(object):
    def __init__(self):
        self._config = None

    def GetConsumerKey(self):
        return self._GetOption("consumer_key")

    def GetConsumerSecret(self):
        return self._GetOption("consumer_secret")

    def GetAccessKey(self):
        return self._GetOption("access_key")

    def GetAccessSecret(self):
        return self._GetOption("access_secret")

    def _GetOption(self, option):
        try:
            return self._GetConfig().get("Tweet", option)
        except:
            return None

    def _GetConfig(self):
        if not self._config:
            self._config = configparser.ConfigParser()
            self._config.read(os.path.expanduser("~/.tweetrc"))
        return self._config


def main():
    try:
        shortflags = "h"
        longflags = [
            "help",
            "consumer-key=",
            "consumer-secret=",
            "access-key=",
            "access-secret=",
            "encoding=",
        ]
        opts, args = getopt.gnu_getopt(sys.argv[1:], shortflags, longflags)
    except getopt.GetoptError:
        PrintUsageAndExit()
    consumer_keyflag = None
    consumer_secretflag = None
    access_keyflag = None
    access_secretflag = None
    encoding = None
    for o, a in opts:
        if o in ("-h", "--help"):
            PrintUsageAndExit()
        if o in ("--consumer-key"):
            consumer_keyflag = a
        if o in ("--consumer-secret"):
            consumer_secretflag = a
        if o in ("--access-key"):
            access_keyflag = a
        if o in ("--access-secret"):
            access_secretflag = a
        if o in ("--encoding"):
            encoding = a
    message = " ".join(args)
    if not message:
        PrintUsageAndExit()
    rc = TweetRc()
    consumer_key = (
        consumer_keyflag or GetConsumerKeyEnv() or rc.GetConsumerKey()
    )
    consumer_secret = (
        consumer_secretflag or GetConsumerSecretEnv() or rc.GetConsumerSecret()
    )
    access_key = access_keyflag or GetAccessKeyEnv() or rc.GetAccessKey()
    access_secret = (
        access_secretflag or GetAccessSecretEnv() or rc.GetAccessSecret()
    )
    if (
        not consumer_key
        or not consumer_secret
        or not access_key
        or not access_secret
    ):
        PrintUsageAndExit()
    api = twitter.Api(
        consumer_key=consumer_key,
        consumer_secret=consumer_secret,
        access_token_key=access_key,
        access_token_secret=access_secret,
        input_encoding=encoding,
    )
    try:
        status = api.PostUpdate(message)
    except UnicodeDecodeError:
        print(
            "Your message could not be encoded.  Perhaps it contains non-ASCII characters? "
        )
        print(
            "Try explicitly specifying the encoding with the --encoding flag"
        )
        sys.exit(2)

    print("{0} just posted: {1}".format(status.user.name, status.text))


if __name__ == "__main__":
    main()



# remove noise from text
# code by Pablo Salvador Lopez
# from https://towardsdatascience.com/distributed-text-preprocessing-with-python-and-dask-5312a4d26ae
from nltk.corpus import stopwords


def remove_noise(text):

    text = re.sub(r",", "", text)
    text = re.sub(r"\w+\d+", " numbers", text)
    text = re.sub(r'\d+', 'numbers', text)
    text = re.sub(r"\$", "dollar ", text)
    text = re.sub(r"\$+", "dollar ", text)
    text = re.sub(r"dollars", "dollar", text)
    text = re.sub(r"[^A-Za-z0-9^,!.\/'+-=]", " ", text)
    text = re.sub(r"!", " ", text)
    text = re.sub(r"what's", "what is ", text)
    text = re.sub(r"\'s", " ", text)
    text = re.sub(r"\'ve", " have ", text)
    text = re.sub(r"n't", " not ", text)
    text = re.sub(r"i'm", "i am ", text)
    text = re.sub(r":", "", text)
    text = re.sub(r" :", "", text)
    text = re.sub(r"\w+\-\w+", "", text)
    text = re.sub(r" -", "", text)
    text = re.sub(r" s ", "", text)
    text = re.sub(r" - ", "", text)
    text = re.sub(r"\'re", " are ", text)
    text = re.sub(r"\'d", " would ", text)
    text = re.sub(r"\'ll", " will ", text)
    #text = re.sub(r",", "", text)
    #text = re.sub(r"\.", " ", text)
    text = re.sub(r"!", " ! ", text)
    text = re.sub(r"\/", " ", text)
    text = re.sub(r"\^", " ^ ", text)
    text = re.sub(r"\+", " + ", text)
    text = re.sub(r"\-", " - ", text)
    text = re.sub(r"\=", " = ", text)
    text = re.sub(r"'", " ", text)
    text = re.sub(r"(\d+)(k)", r"\g<1>000", text)
    text = re.sub(r":", " : ", text)
    text = re.sub(r" e g ", " eg ", text)
    text = re.sub(r" b g ", " bg ", text)
    text = re.sub(r" u s ", " american ", text)
    text = re.sub(r"\0s", "0", text)
    text = re.sub(r" 9 11 ", "911", text)
    text = re.sub(r"e - mail", "email", text)
    text = re.sub(r"j k", "jk", text)
    text = re.sub(r"\s{2,}", " ", text)
    text = re.sub(r"", "", text)
    return text

    def remove_noise2(text):
    text = text.replace(".", "")
    return text

    def expand_contractions(text, contraction_mapping=CONTRACTION_MAP):

    contractions_pattern = re.compile('({})'.format('|'.join(contraction_mapping.keys())),
                                      flags=re.IGNORECASE | re.DOTALL)

    def expand_match(contraction):
        match = contraction.group(0)
        first_char = match[0]
        expanded_contraction = contraction_mapping.get(match)\
            if contraction_mapping.get(match)\
            else contraction_mapping.get(match.lower())
        expanded_contraction = first_char + expanded_contraction[1:]
        return expanded_contraction

    expanded_text = contractions_pattern.sub(expand_match, text)
    expanded_text = re.sub("'", "", expanded_text)
    return expanded_text


# remove stopwords
stop_words = stopwords.words("english")


def remove_stopwords3(text):
    tokens = tokenizer.tokenize(text)
    filtered_words = [w for w in tokens if len(w) > 2 if not w in stop_words]

    return " ".join(filtered_words)


def clean_text(df):

    df["review"] = df.review.map(lambda review: review.lower()).map(
        remove_noise).map(expand_contractions).map(remove_noise2).map(remove_stopwords3)

    return df



# 'Standard Library'
from dataclasses import dataclass
from os import linesep as NL

# 'package imports'
from autosys.profile.autosys_time_it import time_me

from typing import List


@dataclass
class AsciiChars:
    """ #### Newline constants
            NL      - (os.linesep) system specific Newline
            LF      - Linefeed = Linux / macOS Newline
            CR      - Carriage Return
            CRLF    - CR+LF = Windows Newline
            LFCR    - LF+CR = older mac newline code

        #### Common ASCII constants
            NUL     - NUL character
            BEL     - Audible or visual indicator
            BS      - Backspace character
            TAB     - Horizontal Tab character
            VT      - Vertical tab
            FF      - Form Feed (or clear screen)
            ESC     - Escape

        #### Full List

            NUL: str = "\\x00"
            SOH: str = "\\x01"
            STX: str = "\\x02"
            ETX: str = "\\x03"
            EOT: str = "\\x04"
            ENQ: str = "\\x05"
            ACK: str = "\\x06"
            BEL: str = "\\x07"
            BS: str = "\\x08"
            TAB: str = "\\x09"
            VT: str = "\\x0B"
            FF: str = "\\x0C"
            SO: str = "\\x0E"
            SI: str = "\\x0F"
            """

    NL: str = NL
    LF: str = "\x10"
    CR: str = "\x0D"
    CRLF: str = f"{CR}{LF}"
    LFCR: str = f"{LF}{CR}"

    NUL: str = "\x00"
    SOH: str = "\x01"
    STX: str = "\x02"
    ETX: str = "\x03"
    EOT: str = "\x04"
    ENQ: str = "\x05"
    ACK: str = "\x06"
    BEL: str = "\x07"
    BS: str = "\x08"
    TAB: str = "\x09"
    VT: str = "\x0B"
    FF: str = "\x0C"
    SO: str = "\x0E"
    SI: str = "\x0F"

    ESC: str = "\x1B"

    # ASCII control characters
    NUL: str = "\x00"  # Null character
    SOH: str = "\x01"  # Start of Header
    STX: str = "\x02"  # Start of Text
    ETX: str = "\x03"  # End of Text
    EOT: str = "\x04"  # End of Trans.
    ENQ: str = "\x05"  # Enquiry
    ACK: str = "\x06"  # Acknowledgement
    BEL: str = "\x07"  # Bell
    BS: str = "\x08"  # Backspace
    HT: str = "\x09"  # Horizontal Tab
    LF: str = "\x10"  # Line feed
    VT: str = "\x11"  # Vertical Tab
    FF: str = "\x12"  # Form feed
    CR: str = "\x13"  # Carriage return
    SO: str = "\x14"  # Shift Out
    SI: str = "\x15"  # Shift In
    DLE: str = "\x16"  # Data link escape
    DC1: str = "\x17"  # Device control 1
    DC2: str = "\x18"  # Device control 2
    DC3: str = "\x19"  # Device control 3
    DC4: str = "\x20"  # Device control 4
    NAK: str = "\x21"  # Negative acknowl.
    SYN: str = "\x22"  # Synchronous idle
    ETB: str = "\x23"  # End of trans. block
    CAN: str = "\x24"  # Cancel
    EM: str = "\x25"  # End of medium
    SUB: str = "\x26"  # Substitute
    ESC: str = "\x27"  # Escape
    FS: str = "\x28"  # File separator
    GS: str = "\x29"  # Group separator
    RS: str = "\x30"  # Record separator
    US: str = "\x31"  # Unit separator
    DEL: str = "\u0127"  # Delete

    # extended ASCII
    char128: str = "Ç"
    char129: str = "ü"
    char130: str = "é"
    char131: str = "â"
    char132: str = "ä"
    char133: str = "à"
    char134: str = "å"
    char135: str = "ç"
    char136: str = "ê"
    char137: str = "ë"
    char138: str = "è"
    char139: str = "ï"
    char140: str = "î"
    char141: str = "ì"
    char142: str = "Ä"
    char143: str = "Å"
    char144: str = "É"
    char145: str = "æ"
    char146: str = "Æ"
    char147: str = "ô"
    char148: str = "ö"
    char149: str = "ò"
    char150: str = "û"
    char151: str = "ù"
    char152: str = "ÿ"
    char153: str = "Ö"
    char154: str = "Ü"
    char155: str = "¢"
    char156: str = "£"
    char157: str = "¥"
    char158: str = "₧"
    char159: str = "ƒ"
    char160: str = "á"
    char161: str = "í"
    char162: str = "ó"
    char163: str = "ú"
    char164: str = "ñ"
    char165: str = "Ñ"
    char166: str = "ª"
    char167: str = "º"
    char168: str = "¿"
    char169: str = "⌐"
    char170: str = "¬"
    char171: str = "½"
    char172: str = "¼"
    char173: str = "¡"
    char174: str = "«"
    char175: str = "»"
    char176: str = "░"
    char177: str = "▒"
    char178: str = "▓"
    char179: str = "│"
    char180: str = "┤"
    char181: str = "╡"
    char182: str = "╢"
    char183: str = "╖"
    char184: str = "╕"
    char185: str = "╣"
    char186: str = "║"
    char187: str = "╗"
    char188: str = "╝"
    char189: str = "╜"
    char190: str = "╛"
    char191: str = "┐"
    char192: str = "└"
    char193: str = "┴"
    char194: str = "┬"
    char195: str = "├"
    char196: str = "─"
    char197: str = "┼"
    char198: str = "╞"
    char199: str = "╟"
    char200: str = "╚"
    char201: str = "╔"
    char202: str = "╩"
    char203: str = "╦"
    char204: str = "╠"
    char205: str = "═"
    char206: str = "╬"
    char207: str = "╧"
    char208: str = "╨"
    char209: str = "╤"
    char210: str = "╥"
    char211: str = "╙"
    char212: str = "╘"
    char213: str = "╒"
    char214: str = "╓"
    char215: str = "╫"
    char216: str = "╪"
    char217: str = "┘"
    char218: str = "┌"
    char219: str = "█"
    char220: str = "▄"
    char221: str = "▌"
    char222: str = "▐"
    char223: str = "▀"
    char224: str = "α"
    char225: str = "ß"
    char226: str = "Γ"
    char227: str = "π"
    char228: str = "Σ"
    char229: str = "σ"
    char230: str = "µ"
    char231: str = "τ"
    char232: str = "Φ"
    char233: str = "Θ"
    char234: str = "Ω"
    char235: str = "δ"
    char236: str = "∞"
    char237: str = "φ"
    char238: str = "ε"
    char239: str = "∩"
    char240: str = "≡"
    char241: str = "±"
    char242: str = "≥"
    char243: str = "≤"
    char244: str = "⌠"
    char245: str = "⌡"
    char246: str = "÷"
    char247: str = "≈"
    char248: str = "°"
    char249: str = "∙"
    char250: str = "·"
    char251: str = "√"
    char252: str = "ⁿ"
    char253: str = "²"
    char254: str = "■"
    char255: str = "  	"


def char(i: int) -> (str):
    """ Return a printable character from the 'chr' function. """

    try:
        retval = int(i)
        retval = chr(i)
        return retval if len(retval) else " "
    except:
        return " "


def __fmt_def(i: int) -> (str):
    return f"{chr(i)}: str = '{chr(i)}' # ascii code {i} {hex(i)}h"


def __ascii_definition_list(a: int, b: int) -> (List[str]):
    return [__fmt_def(i) for i in range(a, b)]


def __generate_ascii():
    for i in range(32, 127):
        print()


def _test_f1(i=185, n=50000):
    for i in range(n):
        _ = __fmt_def(i)


if __name__ == "__main__":
    from dis import dis

    print("Test ascii generator and class functions:")
    f = __fmt_def
    dis(f)

    ascii_code: int = 1
    while True:
        print(f(ascii_code))
        ascii_code += 1
        if ascii_code > 40:
            break

# __generate_ascii()
# print(__ascii_definition_list(32, 127))



#!/Users/skeptycal/Documents/coding/python/autosys/autosys/setup_utils/venv/bin/python3

# $Id: rst2odt.py 5839 2009-01-07 19:09:28Z dkuhlman $
# Author: Dave Kuhlman <dkuhlman@rexx.com>
# Copyright: This module has been placed in the public domain.

"""
A front end to the Docutils Publisher, producing OpenOffice documents.
"""

import sys

from docutils.core import default_description, publish_cmdline_to_binary
from docutils.writers.odf_odt import Reader, Writer

try:
    import locale
    locale.setlocale(locale.LC_ALL, '')
except:
    pass



description = ('Generates OpenDocument/OpenOffice/ODF documents from '
               'standalone reStructuredText sources.  ' + default_description)


writer = Writer()
reader = Reader()
output = publish_cmdline_to_binary(reader=reader, writer=writer,
    description=description)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" as_system.py - system utilities and tools for efficient python3 """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal

SET_DEBUG: bool = False

if True:
    import os
    import pathlib
    import sys
    import time
    from pathlib import WindowsPath, PosixPath, PurePath, Path
    from typing import Any, Dict, FrozenSet, List, Sequence, Tuple


def _get_env_path():
    """ Return system path """
    return os.getenv("PATH")


def timeit(method):
    def timed(*args, **kw):
        s0 = sys.getsizeof(method)
        t0 = time.time()
        result = method(*args, **kw)
        dt = time.time() - t0
        s1 = sys.getsizeof(method)
        print(method.__name__)
        fn = var_name(kw.get("func"))
        # print(func_name)

        # if 'log_time' in kw:
        #     name = kw.get('log_name', method.__name__.upper())
        #     kw['log_time'][name] = int((dt) * 1000)
        # else:
        print(f"{fn:25.25} - {dt*1000:>6.6} \t{'ms':>3.2}")
        return result

    return timed


class PPath(pathlib.Path):
    """ Base Path Object """

    def __init__(self, pattern: pathlib.PurePath):
        self.pattern = pattern
        if SET_DEBUG:
            print(self.as_uri())
        super().__init__()

    def subs(self, recursive=False):
        """ Yield an iterator of subdirectories. """
        if recursive:
            func = self.rglob(self.pattern + "/")
        else:
            func = self.glob(self.pattern + "/")
        for _ in func:
            # if x.is_dir():
            yield _

    def exists(self):
        return self.exists()

    def is_dir(self):
        return self.is_dir()

    def next_line(self):
        try:
            with self.open() as f:
                return f.readline()
        except OSError as e:
            return e

    def get_name(self) -> str:
        return self.parts[0]

    def str(self):
        return str(self)

    def ls(self, args):
        for x in self.iterdir():
            print(x)


def _get_builtins() -> Sequence[str]:
    return sys.builtin_module_names


def _get_basename(filename: str) -> str:
    """ get only basename from full path """
    return Path(filename).resolve().parts()[-1]  # pathlib version
    # return os.path.basename(filename) # os.path version


def py_path() -> List[str]:
    """ Return list of current python path elements. """
    try:
        return os.environ["PYTHONPATH"].split(os.pathsep)
    except KeyError:
        return []


def py3up() -> bool:
    """ Return True if 'Python >= 3' else False

        If you want to detect pre-Python 3 and don't want to import anything,
        you can (ab)use list comprehension scoping changes """
    # https://stackoverflow.com/questions/1093322/how-do-i-check-what-version-of-python-is-running-my-script/35294211
    # https://stackoverflow.com/a/52825819/9878098
    return (lambda x: [x for x in [False]] and None or x)(True)


def pyver() -> str:
    """ Returns string with python version number in major.minor.micro format.
            (e.g. 3.7.3  or  2.7.12) """
    return ".".join(str(i) for i in __import__("sys").version_info[:3])


def py_shell() -> str:
    """ Returns string containing current python shell name.
            (e.g. ipython notebook, pypy, jupyter notebook, ipython, cpython) """

    shell: str = "unknown"
    PY_ENV = os.environ
    PY_BASE = os.path.basename(PY_ENV["_"])
    if "JPY_PARENT_PID" in PY_ENV:
        shell = "ipython notebook"
    elif "pypy" in PY_ENV:
        shell = "pypy"
    elif "jupyter-notebook" in PY_BASE:
        shell = "jupyter notebook"
    elif "ipython" in PY_BASE:
        shell = "ipython"
    else:
        try:
            import platform

            shell = platform.python_implementation()
        except ImportError:
            pass
    # print("pyshell() output: ", shell.strip())
    return shell.strip()


def _pprint_globals():
    """ Pretty Print all global variables.
        Designed for debugging purposes. """
    print()
    print("Globals: ")
    print("*" * 40)
    width = max(len(i) for i in globals())
    print(f"(key width: {width})")
    for s in globals():
        print("{:<15.15} : {:<64.64}".format(s, str(globals().get(s))))
    print()


def iterable(obj):
    return hasattr(obj, "__iter__") or hasattr(obj, "__getitem__")


# def njoin(l: List[str]) -> str:
#     return '\n'.join(l)


@timeit  # testing efficiency of version 1
def time_iter_check1(n: int = 10000, var: object = None):
    if not var:
        var = "*" * 2000
    for _ in range(n):
        result = iterable(var)
    return result


@timeit  # testing efficiency of version 2
def time_iter_check2(n: int = 10000, var: object = None):
    if not var:
        var = "*" * 2000

    for _ in range(n):
        try:
            result = iter(var)
            result = True
        except TypeError:
            result = False


def time_iter_check(n: int = 10000, var: object = None):
    time_iter_check1(n, var)
    time_iter_check2(n, var)


def njoin(s, delimeter: str = ",") -> str:
    """ Return a string of lines (with LF) from a delimited string or iterable object. """
    if isinstance(s, str):
        return "\n".join(s.split(delimeter))
    try:
        _ = iter(s)
        return "\n".join(_)
    except TypeError:
        pass


def _pprint_code_tests(tests: List[Exception]):
    pass


def _get_functions():
    pass


def _pprint_dict_table(
    data: Dict[Any, Any] = globals(), title: str = "", borders: str = "text"
):
    """ Pretty Print dictionary in table format.
        parameters:
            data: Dict[Any, Any] - dictionary containing table data
            title: str - Displayed Title of Data Table (Title Case Used)
            borders: str - border format (single, double, graphic, text*) """
    if title == "":
        title = data.__repr__()
    print(data.__repr__())
    print(data.__str__())
    maxwidth: int = 75  # 80 - 1 for each outer border and 3 for center padding
    width: int = max(len(i) for i in data) + 1
    width2: int = maxwidth - width
    if SET_DEBUG:
        print()
        print("Title: ", title.title())
        print("*" * len(title))
        print("width: ", width)
        print("width2: ", width2)

    fmt = (
        "{:<"
        + str(width)
        + "."
        + str(width)
        + "} : "
        + "{:<"
        + str(width2)
        + "."
        + str(width2)
        + "}"
    )

    print("*" * 80)
    for s in globals():
        print(fmt.format(s, str(globals().get(s))))
    print("*" * 80)


def _execute_test_code(tests: List[str]) -> List[Exception]:
    result: List[Exception] = []
    for test in tests:
        try:
            # run test code
            print("=> ", test)
            exec(test)
        except Exception as e:
            result.append(e)
    return result


def _run_tests(tests: List[str] = []) -> str:
    if tests == []:
        tests: List[str] = [
            "_pprint_globals()",
            'print("basename: ", _get_basename(__file__))',
            "print(1/0)",
        ]

    results = _execute_test_code(tests)
    for e in results:
        print("e.__class__.__name__", e.__class__.__name__)
        # log.exception(e)
        print("e: ", e)
        print("e.args: ", e.args)
        print("type(e): ", type(e))


if __name__ == "__main__":
    # log = logging.getLogger()
    print(_run_tests())
    print()
    print("... Tests Complete.")
    _pprint_dict_table

    print(njoin(_get_builtins()))
    print()
    print(njoin(_get_env_path(), ":"))
    print(iterable("this"))
    print(iterable(5343))
    print(iterable([1, 2, 3, 4]))
    print(iterable(["a", "b"]))
    s1 = set(x for x in "this")
    print(iterable(s1))
    print(iterable(pathlib.Path()))
    print(time_iter_check(10000000))



#!/usr/bin/env python


class TwitterError(Exception):
    """Base class for Twitter errors"""

    @property
    def message(self):
        """Returns the first argument used to construct this error."""
        return self.args[0]


class PythonTwitterDeprecationWarning(DeprecationWarning):
    """Base class for python-twitter deprecation warnings"""

    pass


class PythonTwitterDeprecationWarning330(PythonTwitterDeprecationWarning):
    """Warning for features to be removed in version 3.3.0"""

    pass


class PythonTwitterDeprecationWarning340(PythonTwitterDeprecationWarning):
    """Warning for features to be removed in version 3.4.0"""

    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
''' wsgiref_sample.py - This is a working “Hello World” WSGI application
    Reference: https://docs.python.org/3/library/wsgiref.html
    '''

from wsgiref.simple_server import make_server

# Every WSGI application must have an application object - a callable
# object that accepts two arguments. For that purpose, we're going to
# use a function (note that you're not limited to a function, you can
# use a class for example). The first argument passed to the function
# is a dictionary containing CGI-style environment variables and the
# second variable is the callable object.


def hello_world_app(environ, start_response):
    status = '200 OK'  # HTTP Status
    headers = [('Content-type', 'text/plain; charset=utf-8')]  # HTTP Headers
    start_response(status, headers)

    # The returned object is going to be printed
    return [b"Hello World\n\nMelanie is cute."]


with make_server('', 8000, hello_world_app) as httpd:
    print("Serving on port 8000...")

    # Serve until process is killed
    httpd.serve_forever()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" # TODO -- @update `AutoSys` package
        copyright (c) 2018 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """
import json

from dataclasses import Field, asdict, dataclass, field
from os import linesep as NL

from typing import Dict, Iterable, List


def roman():
    pass


@dataclass
class Indent:
    """ Return a foolproof indent string. Indents may include f-strings with
            function output, markdown notation, html tags, css labels, etc.

            To set a static indent string, including f-strings with functions
            or indent_sets, use the form:

            ```
            # general form:
            Indent(ind=f"xxx{func(args)}xxx")

            # example
            def count(n=100):
                for i in range(n):
                    yield i

            ind = Indent(ind=f"#{count()}.")

            ```
            where `func` is the function evaluated elsewhere and `x` is static text portions
            if you wish to use `Indent` to evaluate the function (inlining it),
            use:
            ```
            Indent(ind=f"xxx{eval(func(args))}xxx")



            Return the first successful of these values in this order:
            - first `n` characters in 'c' (0 means *all* of `c`)
            - first `n` chars of `DEFAULT_INDENT_STRING`
            - first `DEFAULT_INDENT_NUMBER` chars of `DEFAULT_INDENT_STRING`
            - the default `DEFAULT_INDENT_STRING`
            - 2 spaces

            This allows useful (and recursive) indents like this:
            ```
            $ xxxx
            xxxx
            - xxxx
            *   xxxx
            I.   xxxx
            a.   xxxx
            1)   xxxx
            f"{_get_next_indent(level=_current_indent_level)}"
            ```
            """

    DEFAULT_INDENT_NUMBER: int = 2
    DEFAULT_INDENT_STRING: str = " " * 12

    n: Field = field(default=DEFAULT_INDENT_NUMBER)
    c: Field = field(default=DEFAULT_INDENT_STRING)
    right_aligned: Field = field(default=False)
    repeat: Field = field(default=False)
    ind: Field = field(default="")
    indent_list: Field = field(default_factory=tuple)

    # flag to ignore `n` and `c` ... eval `ind` directly
    is_eval: Field = field(default=False, init=False)
    has_indent_set: Field = field(default=False, init=False)

    def __post_init__(self):
        """ Setup `self.ind` so that it is ready to return. """
        self._setup_ind()
        self.indent_list = ("I.", "A.", "1.", "a.")

    def _setup_ind(self):
        """ Setup `self.ind` so that it is ready to return. """
        if self.ind:  # ignore `c` and `n`
            self.is_eval = True
        else:
            try:
                self.c = str(self.c)
            except:
                self.c = self.DEFAULT_INDENT_STRING

            try:
                self.n = int(self.n)
            except:
                self.n = self.DEFAULT_INDENT_NUMBER

            if 1 < len(self.c) < 81:  # check `c`
                self.c = self.DEFAULT_INDENT_STRING
            if not (0 < self.n < 81):  # check `n`
                self.n = self.DEFAULT_INDENT_NUMBER

            if self.repeat:  # repeat ignores slicing
                self.ind = self.c * self.n
            elif not self.right_aligned:  # slice left
                self.ind = self.ind[:self.n]
            else:  # slice right
                self.ind = self.ind[-self.n:]

    def __str__(self) -> (str):
        """ Return the current indent string. """
        if self.has_indent_set:
            return self.return_next_indent()
        elif self.is_eval:
            return f"{eval(self.ind)}"
        else:
            try:
                if self.n == 0:
                    return self.c
                else:
                    return self.c[:self.n]
            except:
                try:
                    return DEFAULT_INDENT_STRING[:self.n]
                except:
                    try:
                        return DEFAULT_INDENT_STRING[:DEFAULT_INDENT_NUMBER]
                    except:
                        try:
                            return DEFAULT_INDENT_STRING
                        except:
                            pass
            return "  "


def pretty_list(lst: List, indent="  ") -> (str):
    def _str_gen(x: Iterable, indent=indent):
        return (f"{indent}{str(i)}," for i in x)

    return f"[{NL}{NL.join(_str_gen(lst))}{NL}],"


def pretty_dict(dct: Dict, indent="  "):
    def _str_gen(x: Dict, indent=indent):
        return (f"{indent}{k}: {repr(v)}," for k, v in x.items())

    return f"{{{NL}{NL.join(_str_gen(dct))}{NL}}},"


def transpose_list(lst: List):
    return [list(i) for i in zip(*x)]


x = [[31, 17], [40, 51], [13, 12]]

# ind = Indent(n=2, c="------>", right_aligned=True)

# print(pretty_dict(asdict(ind), indent=f"{ind}"))

# print(x)
# print(transpose_list(x))
print()
# print(pretty_list(x, indent=ind))
print()

# print(pretty_list(transpose_list(x)))

# ind = Indent(n=3, c="->  ")


# for i in range(35, 75):
#     print(f"{ind}{chr(i)}")
def count(n=100):
    for i in range(n):
        yield i


ind = Indent(ind="'#{count(i)}.'")

print(pretty_dict(vars(ind)))

print(ind.__init__)

for i in count(5):
    print(ind)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
""" implore.py - An even more desperate way to make requests for Humans.™ """
""" GOALS:
    By creating this app, I wanted to answer these questions:

    - What are the internet activities that I use or want most often?
    - What are the things that are feasible, but that I cannot easily do?

    Here is a list of items I came up with. I have solved them to some degree, but any advice or suggestions is welcome. See Contributors document for information.

    - Get a webpage
    - Modify CSS
    - Convert to a 3d object
        - front is main page
        - shape / depth controlled by varying functions
    - Collect links, images, tags, or something else from webpages
    - Create a 'map' of link destinations
    - Create an image catalog for a specific site(with CSS)
    - Interact with underlying database
    - Disable certain JS apps
    - Disable or interact with certain trackers
    - Watch for cookies and other local data / permissions
    - Block specific activities of specific sites
    - Search for English (or language) words and separate them from tags
    - Analyze metadata
    - Count types of tags and metadata
    - Analyze or avoid ads
    - Test for API functionality
    """


if True:  # Standard Library Dependencies
    from collections import Counter, deque
    from datetime import datetime
    from pprint import pprint
    from time import time, sleep, process_time, perf_counter, gmtime, localtime
    from typing import Any, Dict, List, MutableSequence
    import cgitb
    import os
    import sys
    import traceback

SET_DEBUG: bool = True  # turn on for Dev Debugging
if SET_DEBUG:  # enable detailed logging
    # cgitb.enable([display[, logdir[, context[, format]]]])
    DEFAULT_CGITB_DISPLAY: int = 1  # 0 to suppress messages
    DEFAULT_CGITB_FILE_LOGDIR: str = os.path.join(
        os.path.dirname(__file__), "LOGS"
    )
    DEFAULT_CGITB_CONTEXT: int = 3  # 5 is the normal default
    DEFAULT_CGITB_FORMAT: str = "html"

    cgitb.enable(
        logdir=DEFAULT_CGITB_FILE_LOGDIR,
        display=DEFAULT_CGITB_DISPLAY,
        context=DEFAULT_CGITB_CONTEXT,
        format=DEFAULT_CGITB_FORMAT,
    )

    DEFAULT_CGITB_ERROR: sys.exc_info = ()

if True:  # External Dependencies
    from apscheduler.schedulers.blocking import BlockingScheduler
    from bs4 import BeautifulSoup
    import requests

    try:
        import ujson as json  # faster json if available
    except:
        import json

if True:  # setup html parser
    DEFAULT_PARSER: str = ""
    XML: bool = False  # whether XML parsing is available at all
    try:  # use <lxml> for speed (HTML and / or XML) (VERY fast)
        import lxml as parser

        DEFAULT_PARSER = "lxml"
        XML = True
        XML_PARSER: str = "xml"
    except:
        try:  # use <html5lib> for accuracy (VERY slow)
            import html5lib as parser

            DEFAULT_PARSER = "html5lib"
        except:  # use <html5lib> Python's built-in html parser
            import html.parser as parser

            DEFAULT_PARSER = "html.parser"

if True:  # Default constants
    # * time zones and scheduling
    USE_TZ: bool = False
    sch = BlockingScheduler()

    # * html and parsing
    DEFAULT_ENCODING: str = "UTF-8"
    url: str = ""
    script_text: str = ""
    current_number: str

    now: struct_time = gmtime()


def db_print(*args, file=sys.stderr, **kwargs):
    """ Print messages to STDERR in debug mode (if <SET_DEBUG> is set) """
    if SET_DEBUG:
        print("@db ->", *args, **kwargs)


# contact_list.save_to_file() # with open('list.txt') as p: # p = json.load(p) # print(p)


@app.route("/")
def index():
    pass


def main_test():
    url = "https://www.indeed.com/jobs?q=python&l=Remote"
    matches = soup_match(url, tag_name="div", attrs_pass={"class": "title"})
    # for jobTitle in matches: # if "Developer" in jobTitle.text:
    # text(default_cell_number, 'jobTitle.text')  # break # elif "Jr" in jobTitle.text:
    # print(type(matches)) for match in matches: print(match.text.strip())
    # text(default_cell_number, 'jobTitle.text')  # break


def main():
    db_print(f"{DEFAULT_PARSER}")
    wps = WebPageSet([], check_links=True, maxlen=100)
    wps.append(WebPage("https://www.google.com"))
    wps.append(WebPage("fake page"))
    wps.append("fake page")


if __name__ == "__main__":
    cgitb.enable()
    db_print(cgitb.grey("test grey font"))
    try:
        app.run()
    except:
        pass

    # start_response('200 OK', [('Content-Type', 'text/html')])
    main()


# References
""" Choice of parser

    I use BeautifulSoup to work with html documents.

        Beautiful Soup is a Python library for pulling data out of HTML and XML files. It works with your favorite parser to provide idiomatic ways of navigating, searching, and modifying the parse tree. It commonly saves programmers hours or days of work.

        https://www.crummy.com/software/BeautifulSoup/bs4/doc/

    According to the documentation:

        $ pip install lxml
        $ pip install html5lib

    do not use python's built in html parser (before python 3.2.2).

    - "lxml" - Use for speed (HTML)
    - "xml" - Use for speed (XML)
    - "html5lib" - Best accuracy ... very slow! (HTML)
    - "html.parser" - built in parser ... eh, it works (HTML)
    - lxml is also the ONLY supported XML parser


Latex / Bibtex

    - http://www.bibtex.org/
    - https://libguides.bates.edu/thesis_writers

Shields

    - https://shields.io/
    - https://simpleicons.org/?q=dj
    - https://docs.readthedocs.io/en/stable/badges.html
    - https://img.shields.io/badge/Django-v3-%23092E20?logo=django&color=#339933


    """



# -*- coding: utf-8 -*-


USAGE = """

author    - Michael Treanor  <skeptycal@gmail.com>
copyright - 2019 (c) Michael Treanor
license   - MIT <https://opensource.org/licenses/MIT>
github    - https://www.github.com/skeptycal

Usage: text_colors {demo|version|help}

  Parameters:
      demo, -d, --demo        -- install and initialize
      version, -v, --version  -- display version information
      help, -h, --help        -- display usage and information"""

if __name__ == "__main__":
    print(USAGE)



#!/usr/bin/env python

# 'Standard Library'
import re


class Emoticons:
    POSITIVE = [
        "*O",
        "*-*",
        "*O*",
        "*o*",
        "* *",
        ":P",
        ":D",
        ":d",
        ":p",
        ";P",
        ";D",
        ";d",
        ";p",
        ":-)",
        ";-)",
        ":=)",
        ";=)",
        ":<)",
        ":>)",
        ";>)",
        ";=)",
        "=}",
        ":)",
        "(:;)",
        "(;",
        ":}",
        "{:",
        ";}",
        "{;:]",
        "[;",
        ":')",
        ";')",
        ":-3",
        "{;",
        ":]",
        ";-3",
        ":-x",
        ";-x",
        ":-X",
        ";-X",
        ":-}",
        ";-=}",
        ":-]",
        ";-]",
        ":-.)",
        "^_^",
        "^-^",
    ]

    NEGATIVE = [
        ":(",
        ";(",
        ":'(",
        "=(",
        "={",
        "):",
        ");",
        ")':",
        ")';",
        ")=",
        "}=",
        ";-{{",
        ";-{",
        ":-{{",
        ":-{",
        ":-(",
        ";-(",
        ":,)",
        ":'{",
        "[:",
        ";]",
    ]


class ParseTweet(object):
    # compile once on import
    regexp = {
        "RT": "^RT",
        "MT": r"^MT",
        "ALNUM": r"(@[a-zA-Z0-9_]+)",
        "HASHTAG": r"(#[\w\d]+)",
        "URL": r"([https://|http://]?[a-zA-Z\d\/]+[\.]+[a-zA-Z\d\/\.]+)",
        "SPACES": r"\s+",
    }
    regexp = dict((key, re.compile(value)) for key, value in regexp.items())

    def __init__(self, timeline_owner, tweet):
        """ timeline_owner : twitter handle of user account. tweet - 140 chars from feed; object does all computation on construction
            properties:
            RT, MT - boolean
            URLs - list of URL
            Hashtags - list of tags
        """
        self.Owner = timeline_owner
        self.tweet = tweet
        self.UserHandles = ParseTweet.getUserHandles(tweet)
        self.Hashtags = ParseTweet.getHashtags(tweet)
        self.URLs = ParseTweet.getURLs(tweet)
        self.RT = ParseTweet.getAttributeRT(tweet)
        self.MT = ParseTweet.getAttributeMT(tweet)
        self.Emoticon = ParseTweet.getAttributeEmoticon(tweet)

        # additional intelligence
        if self.RT and len(self.UserHandles) > 0:  # change the owner of tweet?
            self.Owner = self.UserHandles[0]
        return

    def __str__(self):
        """ for display method """
        return (
            "owner %s, urls: %d, hashtags %d, user_handles %d, len_tweet %d, RT = %s, MT = %s"
            % (
                self.Owner,
                len(self.URLs),
                len(self.Hashtags),
                len(self.UserHandles),
                len(self.tweet),
                self.RT,
                self.MT,
            )
        )

    @staticmethod
    def getAttributeEmoticon(tweet):
        """ see if tweet is contains any emoticons, +ve, -ve or neutral """
        emoji = list()
        for tok in re.split(ParseTweet.regexp["SPACES"], tweet.strip()):
            if tok in Emoticons.POSITIVE:
                emoji.append(tok)
                continue
            if tok in Emoticons.NEGATIVE:
                emoji.append(tok)
        return emoji

    @staticmethod
    def getAttributeRT(tweet):
        """ see if tweet is a RT """
        return re.search(ParseTweet.regexp["RT"], tweet.strip()) is not None

    @staticmethod
    def getAttributeMT(tweet):
        """ see if tweet is a MT """
        return re.search(ParseTweet.regexp["MT"], tweet.strip()) is not None

    @staticmethod
    def getUserHandles(tweet):
        """ given a tweet we try and extract all user handles in order of occurrence"""
        return re.findall(ParseTweet.regexp["ALNUM"], tweet)

    @staticmethod
    def getHashtags(tweet):
        """ return all hashtags"""
        return re.findall(ParseTweet.regexp["HASHTAG"], tweet)

    @staticmethod
    def getURLs(tweet):
        r""" URL : [http://]?[\w\.?/]+"""
        return re.findall(ParseTweet.regexp["URL"], tweet)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" supports_color - terminal color detection and setup

    Part of the `AutoSys` package - utilities for macOS apps
        copyright (c) 2019 Michael Treanor
        https://www.github.com/skeptycal/autosys
        https://www.twitter.com/skeptycal

    `AutoSys` is licensed under the `MIT License
        `<https://opensource.org/licenses/MIT>`
    """

# 'Standard Library'
from os import environ as ENV
from platform import platform as PLATFORM
from sys import path as PYTHONPATH, stderr, stdout

from typing import Dict, List


def _SUPPORTS_COLOR() -> (bool):
    # generic script level stderr output characteristics
    _IS_A_TTY: bool = stdout.isatty() and hasattr(stdout, "isatty")
    _IS_PPC: bool = PLATFORM() == "Pocket PC"
    _IS_WIN32: bool = PLATFORM() == "win32"
    IS_WIN: bool = "win" in PLATFORM().lower()
    _IS_ANSICON: bool = "ANSICON" in ENV
    _IS_WIN_COLOR: bool = _IS_WIN32 and _IS_ANSICON
    _IS_EDGE_CASE: bool = _IS_WIN_COLOR and not _IS_PPC
    retval = _IS_A_TTY or _IS_EDGE_CASE
    return retval


SUPPORTS_COLOR: bool = _SUPPORTS_COLOR()
print(SUPPORTS_COLOR)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" py_globals.py """
# copyright (c) 2019 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal


def print_all_vars():
    for var in globals():
        # print("%s = %s" % (k, repr(v)))
        print("%s => %s" % (var, globals()[var]))


def get_var_def(v):
    return globals().values() == v


def get_var_value(v):
    # this is a redundant function for testing ... just call 'v' if you want the value of v!
    # it does allow the setting of a default value, which may be handy at times?
    return globals().get(v, "IDK")


def create_dict_table(*dict, color: str = "") -> str:
    """ Generate a table with ansi borders from a Python dictionary """
    """ works for 2 column Python tables, with width given, at this point """


a = "this is a"
b = "b has a number 32"
print("%(a)s    %(b)s" % globals())
print()
# print(globals())
print("return all globals: ")
print(print_all_vars())
print()
print(
    "return the value of the variable who's name is '__doc__': ",
    get_var_value("__doc__"),
)
print(
    "return the value of the variable who's name is 'pyglobals.py': ",
    get_var_value("pyglobals.py"),
)
print(get_var_def("__doc__"))
print(get_var_def("pyglobals.py"))



# Ref: https://stackoverflow.com/a/16700993/9878098
# delattr() is what you want. Loop through vars() of the class and test for attribute names starting with "test_". E.g.,


class Dummy:
    @classmethod
    def __vars(cls):
        return list(vars(cls))

    @classmethod
    def remove_prefixed_methods(cls, prefix: str = "test_"):
        for name in cls.__vars():
            if name.startswith(prefix) and callable(getattr(cls, name)):
                print(name)
                # delattr(cls, name)


d = Dummy()
d.remove_prefixed_methods(prefix="")
print("*" * 40)


# I'd advise against using dir(), as this will show you names from your parent classes as well, so not all the names you get from dir() may defined on the class you're targeting.



def getdefaultlocale(envvars=("LC_ALL", "LC_CTYPE", "LANG", "LANGUAGE")):

    """ Tries to determine the default locale settings and returns
        them as tuple (language code, encoding).

        According to POSIX, a program which has not called
        setlocale(LC_ALL, "") runs using the portable 'C' locale.
        Calling setlocale(LC_ALL, "") lets it use the default locale as
        defined by the LANG variable. Since we don't want to interfere
        with the current locale setting we thus emulate the behavior
        in the way described above.

        To maintain compatibility with other platforms, not only the
        LANG variable is tested, but a list of variables given as
        envvars parameter. The first found to be defined will be
        used. envvars defaults to the search path used in GNU gettext;
        it must always contain the variable name 'LANG'.

        Except for the code 'C', the language code corresponds to RFC
        1766.  code and encoding can be None in case the values cannot
        be determined.

    """

    try:
        # check if it's supported by the _locale module
        import _locale

        code, encoding = _locale._getdefaultlocale()
    except (ImportError, AttributeError):
        pass
    else:
        # make sure the code/encoding values are valid
        if sys.platform == "win32" and code and code[:2] == "0x":
            # map windows language identifier to language name
            code = windows_locale.get(int(code, 0))
        # ...add other platform-specific processing here, if
        # necessary...
        return code, encoding

    # fall back on POSIX behaviour
    import os

    lookup = os.environ.get
    for variable in envvars:
        localename = lookup(variable, None)
        if localename:
            if variable == "LANGUAGE":
                localename = localename.split(":")[0]
            break
    else:
        localename = "C"
    return _parse_localename(localename)



#!/usr/bin/env python3
# -*- encoding: utf-8 -*-
""" anansi.py - Tricky and fun ansi text utilities for python programs.
        import anansi or use with the following CLI syntax:

    Usage:
        - anansi TEXT [-z]
        - anansi FILE(s) [-Rz] [-q | -v ] [--pattern PATTERN]
        - anansi TEMPLATE [-Rz] [-q | -v] [--pattern PATTERN]
        - anansi [--help | --version]

    Options:
        FILE(s)                 Search pattern to match
        -q, --quiet             Suppress most error messages  [default: True]
        -r, --recursive         Perform search recursively    [default: False]
        -v, --verbose           Display detailed progress     [default: False]
        -z, --zero              End each output line with NUL [default: False]

        -P PATTERN --pattern PATTERN    Pattern to highlight  [default: None]

        --version               Show version.
        --debug                 Show debug info and test results.
        -h, --help              Show this screen.

    Exit status:

        0 if all file names were printed without issue.
        1 otherwise.

    Based on ANSI standard ECMA-48:
    http://www.ecma-international.org/publications/files/ECMA-ST/Ecma-048.pdf
    """
"""               Anansi, A Bit of Lore

    # - named after Anansi, the trickster, of West African and Caribbean
    #   folklore. Before Anansi, there were no stories in the world. What
    #   a terrible and ignorant place that must have been!

    # It was Anansi who convinced Nyame, The Sky-God, to share his stories
    #   with the world, but only after capturing the Python, the Hornets,
    #   the Leopard, and the Fairy.
    """

if True:  # !------------------------ imports
    # import re
    # import sys
    # from sys import argv
    # from platform import platform as PLATFORM
    from time import sleep
    from autosys import _version as VERSION

    # from autosys.debug import dbprint, db_column_ruler
    # from autosys.cli.terminal import *


    class Ansi(str):

        # !------------------------ ANSI formatting constants
        # - ANSI ECMA-48 Control Sequence Introducer
        CSI: str = f"{ESC}["
        ANSI_SEP: str = ";"  # - ANSI ECMA-48 default separator

        # - {n} - format string for basic 7 bit ansi escape codes
        FMT_CSI: str = f"{CSI}{{}}m"
        # format for 7 bit foreground (0-7)
        FMT_7BIT_FG: str = f"{CSI}3{{}}m"
        # format for 7 bit background (0-7)
        FMT_7BIT_BG: str = f"{CSI}4{{}}m"
        # format for 7 bit bright foreground (0-7)
        FMT_7BIT_FG_BRIGHT: str = f"{CSI}9{{}}m"
        # format for 7 bit bright foreground (0-7)
        FMT_7BIT_BG_BRIGHT: str = f"{CSI}10{{}}m"

        # - {3,4}{0-255} - format string for 256 color codes
        # - 3 is foreground; 4 is background
        #
        # ESC[38; 5; ⟨n⟩ m Select foreground color
        # ESC[48; 5; ⟨n⟩ m Select background color
        # - format string for 256 color codes
        FMT_CSI_8BIT: str = f"{CSI}{{}}8;5;{{}}m"
        # - format string for 8 bit foreground
        FMT_8BIT_FG: str = f"{CSI}38;5;{{}}m"
        # - format string for 8 bit background
        FMT_8BIT_BG: str = f"{CSI}48;5;{{}}m"

        # ESC[ 38;2;⟨r⟩;⟨g⟩;⟨b⟩ m Select RGB foreground color
        # ESC[ 48;2;⟨r⟩;⟨g⟩;⟨b⟩ m Select RGB background color
        # - use R;G;B to select foreground colors
        CSI_24BITFG: str = f"{CSI}[38;2;{{}};{{}};{{}}m"
        # - use R;G;B to select background colors
        CSI_24BITBG: str = f"{CSI}[48;2;{{}};{{}};{{}}m"
        SUFFIX: str = "m"  # - suffix for ansi codes
        # !------------------------ ANSI regex constants
        RE_CSI = r"\x1B\["  # - regex string for CSI

        # 7 bit C1 ANSI sequences
        ANSI_ESCAPE_7BIT = re.compile(
            r"""
            \x1B  # ESC
            (?:   # 7 bit C1 Fe (except CSI)
                [@-Z\\-_]
            |     # or [ for CSI, followed by a control sequence
                \[
                [0-?]*  # Parameter bytes
                [ -/]*  # Intermediate bytes
                [@-~]   # Final byte
            )
        """,
            re.VERBOSE,
        )  # ??? untested [1]

        def _un_ansi(self, n: str) -> str:
            """ Return string with 7 bit ANSI escape sequences removed. """
            return ANSI_ESCAPE_7BIT.sub("", n)

        ANSI_ESCAPE_8BIT = re.compile(
            r"""(\x9B|\x1B\[)[0-?]*[ -\/]*[@-~]""")  # ??? untested [2]

        ANSI_ESCAPE = re.compile(r"(?:\x1B[@-_]|[\x80-\x9F])[0-?]*[ -/]*[@-~]")

        def remove_needle():
            pass

        def escape_ansi(self, needle: str):
            return self.ANSI_ESCAPE.sub("", needle)

        # !------------------------ ANSI default constants

        DEFAULT_FG_CODE: str = "229"
        DEFAULT_BG_CODE: str = "0"
        DEFAULT_EFFECT_CODE: str = "0"
        DEFAULT_FG: str = FMT_8BIT_FG.format(DEFAULT_FG_CODE)
        DEFAULT_BG: str = FMT_8BIT_BG.format(DEFAULT_BG_CODE)
        DEFAULT_ANSI_TEXT: str = f"{DEFAULT_EFFECT_CODE}{DEFAULT_FG}{DEFAULT_BG}"
        FG_DEFAULT: str = f"{DEFAULT_EFFECT_CODE}{DEFAULT_BG}{{}}"

        # !------------------------ ANSI class setup

        def __init__(self):
            super().__init__()
            self._add_8bit()

        def _add_dynamic_method(self, name, value):
            """ Add dynamic method to class. """
            def key_method(self, value=value):
                """ Dynamic method that returns <value>. """
                return value

            try:
                setattr(self.__class__, name, value)
            except Exception:
                pass

        def __getattribute__(self, name):
            # if SUPPORTS_COLOR:
            #     return ''
            return super().__getattribute__(name)
            # self.__getattribute__()

        # @lru_cache
        def _add_8bit(self):
            """ Create dynamic color methods for each ANSI 256 color codes, both foreground and background. """
            s: str = "COLOR"
            for i in range(255):
                if i == 232:
                    s = "GREY"
                self._add_dynamic_method(f"{s}{i}", self.FMT_8BIT_FG.format(i))
                self._add_dynamic_method(f"BG_{s}{i}",
                                         self.FMT_8BIT_BG.format(i))

        # def __iter__(self):
        #     yield from dataclasses.astuple(self)

        def __add__(self, value) -> str:
            try:
                return value + self
            except:
                return self

        def __mul__(self, value: int) -> str:
            try:
                return self * value
            except:
                return self

        def __contains__(self, key):
            return super().__contains__(key)

        def BOLD(self) -> str:
            try:
                self = self.B + self
            except Exception as e:
                dbprint(e)
            return self

        # !------------------------ encode ANSI color codes

        @lru_cache()
        def encode_color_str(
            self,
            fg=DEFAULT_FG_CODE,
            bg=DEFAULT_BG_CODE,
            ef=DEFAULT_EFFECT_CODE,
        ) -> str:
            if SUPPORTS_COLOR:
                return f"{self.ef(ef)}{self.bg(bg)}{self.fg(fg)}"
            else:
                return ""

        @lru_cache()
        def encode_color_tuple(
            self,
            fg=DEFAULT_FG_CODE,
            bg=DEFAULT_BG_CODE,
            ef=DEFAULT_EFFECT_CODE,
        ) -> Tuple[str, ]:
            return tuple(self.effect(ef), self.bg(bg), self.fg(fg))

        @lru_cache()
        def fg_default(self, c: str) -> str:
            """ Return string with default effect, default background, and
                forground color string <c> encoded.

                c: str - add any foreground string
                DEFAULT_EFFECT_CODE, DEFAULT_BG - added by default
                """
            if SUPPORTS_COLOR:
                return self.encode_color_str(self.fg(c))
            else:
                return ""

        @lru_cache()
        def effect(self, ef: int = 0) -> str:
            """ #### Encode ANSI effect code (default: 0)
                - NONE :            0
                - BOLD :            1
                - FAINT :           2
                - ITALIC :          3
                - UNDERLINE :       4
                - BLINK :           5
                - REVERSE :         7
                - CONCEAL :         8
                - STRIKE :          9
                - FRAME :           51
                - CIRCLE :          52
                - OVERLINE :        53
                """
            if SUPPORTS_COLOR:
                return self.FMT_CSI.format(ef)
            else:
                return ""

        @lru_cache()
        def fg(self, color: int = 15) -> str:
            """ Encode ANSI 8 bit foreground color (default: 15)

                    foreground encoding - `ESC[38;5;3{color}m`

                color -
                - 0-7:     standard colors(e.g. 7-bit ESC[30–37 m)
                - 8-15:    high intensity colors(e.g. 7-bit ESC[90–97 m)
                - 16-231:  6×6×6 cube(216 colors) 16 + 36×r + 6×g + b(0 ≤ r, g, b ≤ 5)
                - 232-255: grayscale from black to white in 24 steps
                """
            if SUPPORTS_COLOR:
                return self.FMT_8BIT_FG.format(color)
            else:
                return ""

        @lru_cache()
        def bg(self, color: int = 0) -> str:
            """ Encode ANSI 8 bit background color (default: 0)

                    background encoding - `ESC[48;5;4{color}m`

                color -
                - 0-7:     standard colors(e.g. 7-bit ESC(40–47 m)
                - 8-15:    high intensity colors(e.g. 7-bit ESC[90–97 m)
                - 16-231:  6×6×6 cube(216 colors) 16 + 36×r + 6×g + b(0 ≤ r, g, b ≤ 5)
                - 232-255: grayscale from black to white in 24 steps
                """
            if SUPPORTS_COLOR:
                return self.FMT_8BIT_BG.format(color)
            else:
                return ""

        @lru_cache()
        def _show_colors(self) -> int:
            if not SUPPORTS_COLOR:
                return -1
            else:
                c: str = "COLOR"
                for i in range(255):
                    if i == 233:
                        c = "GREY"
                    s = f"{c}{i}"
                    s = f"MAIN"
                    s = f"Ansi.{s}"
                    s = s + f"{c}.{s}"
                    print(f"{s:8} ", end="")
                    if i % 8 == 0:
                        print()
                return 0

        # @staticmethod
        # def set_code(self, s: str, file=stdout):
        #     print(s, file=file)

        # !------------------------ ANSI cursor controls

        def move_to(self, L: int, C: int):
            """ Position the Cursor:

                    <ESC>[<L>;<C>H
                        Or
                    <ESC>[<L>;<C>f

                puts the cursor at line L and column C.
                """
            if SUPPORTS_COLOR:
                print(f"\x1B[{L};{C}H")

        def up(self, n: int = 1):
            """ Move the cursor up N lines:

                    <ESC>033[<N>A
                """
            if SUPPORTS_COLOR:
                print(f"\x1B[{n}A")

        def down(self, n: int = 1):
            """ Move the cursor down N lines:

                    <ESC>033[<N>B
                """
            if SUPPORTS_COLOR:
                print(f"\x1B[{n}B")

        def left(self, n: int = 1):
            """ Move the cursor forward N columns:

                    <ESC>033[<N>C
                """
            if SUPPORTS_COLOR:
                print(f"\x1B[{n}C")

        def right(self, n: int = 1):
            """ Move the cursor backward N columns:

                    <ESC>033[<N>D
                """
            if SUPPORTS_COLOR:
                print(f"\x1B[{n}D")

        def clear(self):
            """ Clear the screen, move to (0,0):

                    <ESC>033[2J
                """
            if SUPPORTS_COLOR:
                print(f"\x1B[2J")

        def eol(self):
            """ Erase to end of line:

                    <ESC>033[K
                """
            if SUPPORTS_COLOR:
                print(f"\x1BK")

        def save_cursor(self):
            """ Save cursor position:

                    <ESC>033[s
                """
            if SUPPORTS_COLOR:
                print(f"\x1Bs")

        def restore_cursor(self):
            """ Restore cursor position:

                    <ESC>033[u
                """
            if SUPPORTS_COLOR:
                print(f"\x1Bu")

        def loading(
            self,
            delay: float = 0.1,
            message: str = "Loading ...",
            percent: bool = True,
        ):
            """ ### Terminal progress Indicator

                # Usage:

                    loading(delay: float = 0.1, message: str = 'Loading ...', percent: bool = True )


                    delay - delay between ticks
                    message - feedback shown during countdown (default: 'Loading...')
                    percent - display the '%' symbol

                """
            sp: str = "%"
            if not percent:
                sp = ""
            print(message)
            for i in range(0, 100):
                sleep(delay)
                stdout.write("\x1B[1000D" + str(i + 1) + sp)
                stdout.flush()
            print()

    a = Ansi()


def _main_(args=argv[1:]):
    pass


if __name__ == "__main__":
    _debug_: bool = True
    _main_()
""" # !------------------------  TODO: Ideas and additions:

    1. integrate into <str> class so it works well with others ...
        e.g. output = some_string.capitaize.BLUE.strip

    2. Random changing letter colors in words
        (iterate over letters, interleve random formatting)

    3. Patterned changing letter colors in words
        (zip formatting with letters, join)

    4. Slice formatting options for sentences similar to sometext.format(x, y)
        e.g. some_string.format("1:3, 3, 4:6, 6:", [BLUE, RED, Color248,
        YELLOW]) (break string into slices, prepend formatting, join)

    5. Add common color formats and interfaces (1)
        css parsing, js library, 16bit color, 32bit color, etc.

    6. Add VSCssssssssssssssssssssode integration
    """
""" # !------------------------  References:

    General:

    - Wikipedia - ANSI escape codes - https://en.wikipedia.org/wiki/ANSI_escape_code

        >In 2016, Microsoft released the Windows 10 Version 1511 update which unexpectedly implemented support for ANSI escape sequences.[12] The change was designed to complement the Windows Subsystem for Linux, adding to the Windows Console Host used by Command Prompt support for character escape codes used by terminal-based software for Unix-like systems. This is not the default behavior and must be enabled programmatically with the Win32 API via SetConsoleMode(handle, ENABLE_VIRTUAL_TERMINAL_PROCESSING).[13] This was enabled by CMD.EXE but not initially by PowerShell;[14] however, Windows PowerShell 5.1 now enables this by default. The ability to make a string constant containing ESC was added in PowerShell 6 with (for example) "`e[32m";[15] for PowerShell 5 you had to use [char]0x1B+"[32m".



    - ECMA - http://www.ecma-international.org/publications/standards/Ecma-048.htm

    1. 7 bit CSI regex and great explanation of ANSI codes - https://stackoverflow.com/a/14693789/9878098

    2. 8 bit CSI regex - https://stackoverflow.com/a/33925425


    # Issue with Windows registry setting:
    #   https://stackoverflow.com/a/57154895
    #   File "/Library/Frameworks/Python.framework/Versions/3.8/lib/python3.8/enum.py", line 221, in __new__

    """
""" # !------------------------  Color formatting
    # p1 {background-color: #ff0000;}                /* red in HEX format */
    # p2 {background-color: hsl(120, 100%, 50%);}    /* green in HSL format */
    # p3 {background-color: rgba(0, 4, 255, 0.733);} /* blue with alpha channel in RGBA format */
    """



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" re_bool.py """
# copyright (c) 2020 Michael Treanor
# https://www.github.com/skeptycal
# https://www.twitter.com/skeptycal


class ReBool(int):
    def __new__(cls, value):
        return int.__new__(cls, bool(value))

    def __repr__(self):
        return "MyBool." + ["False", "True"][self]

    def __str__(self):
        return f"bool{self}"


x = ReBool(1)
print(x)
print(ReBool(2) == 1)
print(ReBool(1) == 1)
print(ReBool(2) == 2)
print(ReBool(2))
